"""
See https://en.wikipedia.org/wiki/Magic_square

Examples of Execution:
  python3 MagicSquare.py -data=[4,null]
  python3 MagicSquare.py -data=MagicSquare_example0.txt -dataparser=MagicSquare_Parser.py
"""

from pycsp3 import *

n, clues = data
magic = n * (n * n + 1) // 2

# x[i][j] is the value at row i and column j of the magic square
x = VarArray(size=[n, n], dom=range(1, n * n + 1))

satisfy(
    AllDifferent(x),

    [Sum(row) == magic for row in x],

    [Sum(col) == magic for col in columns(x)],

    # tag(diagonals)
    [Sum(dgn) == magic for dgn in [diagonal_down(x), diagonal_up(x)]],

    # respecting specified clues (if any)  tag(clues)
    [x[i][j] == clues[i][j] for i in range(n) for j in range(n) if clues and clues[i][j] != 0]
)





"""
Problem 044 on CSPLib

Examples of Execution:
  python3 Steiner3.py
  python3 Steiner3.py -data=6
"""

from pycsp3 import *

n = data or 6
nTriples = (n * (n - 1)) // 6

table = {(i1, i2, i3, j1, j2, j3) for (i1, i2, i3, j1, j2, j3) in product(range(1, n + 1), repeat=6) if
         different_values(i1, i2, i3) and different_values(j1, j2, j3) and len({i for i in {i1, i2, i3} if i in {j1, j2, j3}}) <= 1}

# x[i] is the ith triple of values
x = VarArray(size=[nTriples, 3], dom=range(1, n + 1))

satisfy(
    # each triple must be formed of strictly increasing integers
    [Increasing(triple, strict=True) for triple in x],

    # each pair of triples must share at most one value
    [(triple1 + triple2) in table for (triple1, triple2) in combinations(x, 2)]
)



"""
See "Mathematical methods of organizing and planning production" by L. V. Kantorovich, Management Science, 6(4):366–422, 1960
See "From High-Level Model to Branch-and-Price Solution in G12", by J. Puchinger, P. Stuckey, M. Wallace, and S. Brand, CPAIOR 2008: 218-232

Example of Execution:
  python3 Cutstock.py -data=Cutstock_small.json
"""

from pycsp3 import *

nPieces, pieceLength, items = data
lengths, demands = zip(*items)
nItems = len(data.items)

# p[i] is 1 iff the ith piece of the stock is used
p = VarArray(size=nPieces, dom={0, 1})

# r[i][j] is the number of items of type j built using stock piece i
r = VarArray(size=[nPieces, nItems], dom=lambda i, j: range(demands[j] + 1))

satisfy(
    # each item demand must be exactly satisfied
    [Sum(r[:, j]) == demand for j, demand in enumerate(demands)],

    # each piece of the stock cannot provide more than its length
    [r[i] * lengths <= p[i] * pieceLength for i in range(nPieces)],

    # tag(symmetry-breaking)
    [Decreasing(p), LexDecreasing(r)]
)

minimize(
    # minimizing the number of used pieces
    Sum(p)
)



from pycsp3 import *

x = VarArray(size=4, dom=lambda i: range(6) if i < 2 else range(9))

satisfy(
    x[0] + 1 == x[1],
    x[1] + 1 == x[2],
    x[2] + 1 == x[3],
    x[0] + x[1] + x[2] + x[3] == 14
)



"""
Problem 065 on CSPLib

Examples of Execution:
  python3 Opd.py
  python3 Opd.py -data=[4,6,4]
  python3 Opd.py -data=[4,6,4] -variant=aux
"""

from pycsp3 import *

v, b, r = data or (4, 4, 4)

# x[i][j] is the value at row i and column j
x = VarArray(size=[v, b], dom={0, 1})

satisfy(
    # each row sums to 'r'
    Sum(x[i]) == r for i in range(v)
)

if not variant():
    minimize(
        # minimizing the maximum value of dot products between all pairs of distinct rows
        Maximum(x[i] * x[j] for i, j in combinations(range(v), 2))
    )

elif variant("aux"):
    # s[i][j][k] is the scalar variable for the product of x[i][k] and x[j][k]
    s = VarArray(size=[v, v, b], dom=lambda i, j, k: {0, 1} if i < j else None)

    satisfy(
        # computing scalar variables
        s[i][j][k] == x[i][k] * x[j][k] for i, j in combinations(range(v), 2) for k in range(b)
    )

    minimize(
        # minimizing the maximum value of dot products between all pairs of distinct rows
        Maximum(Sum(s[i][j]) for i, j in combinations(range(v), 2))
    )

satisfy(
    # tag(symmetry-breaking)
    LexIncreasing(x, matrix=True)
)



"""
See https://en.wikipedia.org/wiki/Open-shop_scheduling

Example of Execution:
  python3 SchedulingOS.py -data=GP-os-01.json
"""

from pycsp3 import *

durations = data  # durations[i][j] is the duration of operation/machine j for job i
horizon = sum(sum(t) for t in durations) + 1
n, m = len(durations), len(durations[0])

# s[i][j] is the start time of the jth operation for the ith job
s = VarArray(size=[n, m], dom=range(horizon))

# d[i][j] is the duration of the jth operation of the ith job
d = VarArray(size=[n, m], dom=lambda i, j: durations[i])

# mc[i][j] is the machine used for the jth operation of the ith job
mc = VarArray(size=[n, m], dom=range(m))

# sd[i][k] is the start (dual) time of the kth machine when used for the ith job
sd = VarArray(size=[n, m], dom=range(horizon))

satisfy(
    # operations must be ordered on each job
    [Increasing(s[i], lengths=d[i]) for i in range(n)],

    # each machine must be used for each job
    [AllDifferent(mc[i]) for i in range(n)],

    [(mc[i][j], d[i][j]) in enumerate(durations[i]) for j in range(m) for i in range(n)],

    # tag(channeling)
    [sd[i][mc[i][j]] == s[i][j] for j in range(m) for i in range(n)],

    # no overlap on resources
    [NoOverlap(origins=sd[:, j], lengths=durations[:, j]) for j in range(m)],

    # tag(redundant-constraints)
    [s[i][- 1] + d[i][- 1] >= sum(durations[i]) for i in range(n)]
)

minimize(
    # minimizing the makespan
    Maximum(s[i][- 1] + d[i][- 1] for i in range(n))
)



"""
See BPPLIB – A Bin Packing Problem Library

Example of Execution:
  python3 BinPacking.py -data=BinPacking_n1c1w4a.json
"""

from pycsp3 import *

capacity, weights = data  # bin capacity and item weights
nItems = len(weights)


def n_bins():
    cnt = 0
    curr_load = 0
    for i, weight in enumerate(weights):
        curr_load += weight
        if curr_load > capacity:
            cnt += 1
            curr_load = weight
    return cnt


def max_items_per_bin():
    curr = 0
    for i, weight in enumerate(weights):
        curr += weight
        if curr > capacity:
            return i
    return -1


def occurrences_of_weights():
    pairs = []
    cnt = 1
    for i in range(1, nItems):
        if weights[i] != weights[i - 1]:
            pairs.append((weights[i - 1], cnt))
            cnt = 0
        cnt += 1
    pairs.append((weights[-1], cnt))
    return pairs


nBins, maxPerBin = n_bins(), max_items_per_bin()

# x[i][j] is the weight of the jth object put in the ith bin. It is 0 if less than j objects are present in the bin.
x = VarArray(size=[nBins, maxPerBin], dom={0, *weights})

if not variant():
    satisfy(
        # not exceeding the capacity of each bin
        [Sum(x[i]) <= capacity for i in range(nBins)],

        # items are stored decreasingly in each bin according to their weights
        [Decreasing(x[i]) for i in range(nBins)]
    )
elif variant("table"):
    def table():
        def table_recursive(n_stored, i, curr):
            assert len(tuples) < 200000000, "impossible to build a table of moderate size"  # hard coding (value)
            assert curr + weights[i] <= capacity
            tmp[n_stored] = weights[i]
            curr += weights[i]
            tuples.add(tuple(tmp[j] if j < n_stored + 1 else 0 for j in range(maxPerBin)))
            for j in range(i):
                if curr + weights[j] > capacity:
                    break
                if j == i - 1 or weights[j] != weights[j + 1]:
                    table_recursive(n_stored + 1, j, curr)

        tmp = [0] * maxPerBin
        tuples = {tuple(tmp)}
        for i in range(nItems):
            if i == nItems - 1 or weights[i] != weights[i + 1]:
                table_recursive(0, i, 0)
        return tuples


    table = table()
    satisfy(
        x[i] in table for i in range(nBins)
    )

satisfy(
    # ensuring that each item is stored in a bin
    Cardinality(x, occurrences={0: nBins * maxPerBin - nItems} + {wgt: occ for (wgt, occ) in occurrences_of_weights()}),

    # tag(symmetry-breaking)
    LexDecreasing(x)
)

maximize(
    # maximizing the number of unused bins
    Sum(x[i][0] == 0 for i in range(nBins))
)



from pycsp3 import *

# Illustration of how:
#  - to declare stand-alone (symbolic) variables
#  - to post binary table constraints

a, b = "a", "b"  # two symbols

x = Var(a, b)
y = Var(a, b)
z = Var(a, b)

satisfy(
    (x, y) in {(a, a), (b, b)},
    (x, z) in {(a, a), (b, b)},
    (y, z) in {(a, b), (b, a)}
)



from pycsp3 import *

# Illustration of how:
#  - to declare stand-alone (0/1) variables
#  - to post binary table constraints

x = Var(0, 1)
y = Var(0, 1)
z = Var(0, 1)

satisfy(
    (x, y) in {(0, 0), (1, 1)},
    (x, z) in {(0, 0), (1, 1)},
    (y, z) in {(0, 1), (1, 0)}
)



"""
See 'Constraint Solving and Planning with Picat' (page 43)
From Tony Hurlimann, A coin puzzle, SVOR-contest 2007

Some data: (8,4) (8,5) (9,4) (10,4) (31,14)

Examples of Execution:
  python3 CoinsGrid.py
  python3 CoinsGrid.py -data=[10,4]
"""

from pycsp3 import *

n, c = data or (8, 4)

#  x[i][j] is 1 if a coin is placed at row i and column j
x = VarArray(size=[n, n], dom={0, 1})

satisfy(
    [Sum(x[i]) == c for i in range(n)],

    [Sum(x[:, j]) == c for j in range(n)]
)

minimize(
    Sum(x[i][j] * abs(i - j) ** 2 for i in range(n) for j in range(n))
)


#  TODO: other variants in Hurlimann's paper



from pycsp3.problems.data.parsing import *

nWarehouses = number_in(line())
nStores = number_in(next_line())

data["fixedCost"] = number_in(next_line())
if next_line().startswith("capacity"):
    data["warehouseCapacities"] = numbers_in(line())
    next_line()
else:
    data["warehouseCapacities"] = [data.nStores] * data.nWarehouses
data["storeSupplyCosts"] = [numbers for numbers in [numbers_in(line) for line in remaining_lines()] if numbers]

assert nWarehouses == len(data["warehouseCapacities"]) and nStores == len(data["storeSupplyCosts"])



""""
See Problem 014 on CSPLib

Examples of Execution:
  python3 SolitaireBattleship.py -data=SolitaireBattleship_sb-12-12-5-0.json
"""

from pycsp3 import *

fleet, rowSums, colSums, hints = data
surfaces = [ship.size * ship.cnt for ship in fleet]
pos, neg = [ship.size for ship in fleet], [-ship.size for ship in fleet]
n, nTypes = len(colSums), len(pos)


def automaton(horizontal):
    q = Automaton.q  # for building state names
    transitions = [(q(0), 0, q(0)), (q(0), neg if horizontal else pos, "qq"), ("qq", 0, q(0))]
    for i in pos:
        v = i if horizontal else -i
        transitions += [(q(0), v, q(i, 1))] + [(q(i, j), v, q(i, j + 1)) for j in range(1, i)] + [(q(i, i), 0, q(0))]
    return Automaton(start=q(0), final=q(0), transitions=transitions)


horizontal_automaton, vertical_automaton = automaton(True), automaton(False)  # automata for ships online

# s[i][j] is 1 iff the cell at row i and col j is occupied by a ship segment
s = VarArray(size=[n + 2, n + 2], dom={0, 1})

# t[i][j] is 0 iff the cell at row i and col j is unoccupied, the type (size) of the ship fragment otherwise,
# when positive, the ship is put horizontally, when negative, the ship is put vertically
t = VarArray(size=[n + 2, n + 2], dom=set(neg) | {0} | set(pos))

# cp[i] is the number of positive ship segments of type i
cp = VarArray(size=nTypes, dom=range(max(surfaces) + 1))

# cn[i] is the number of negative ship segments of type i
cn = VarArray(size=nTypes, dom=lambda i: {0} if fleet[i].size == 1 else range(max(surfaces) + 1))


def hint_ctr(c, i, j):
    if c == 'w':
        return s[i][j] == 0
    if c in {'c', 'l', 'r', 't', 'b'}:
        return [
            s[i][j] == 1,
            s[i - 1][j] == (1 if c == 'b' else 0),
            s[i + 1][j] == (1 if c == 't' else 0),
            s[i][j - 1] == (1 if c == 'r' else 0),
            s[i][j + 1] == (1 if c == 'l' else 0)
        ]
    if c == 'm':
        return [
            s[i][j] == 1,
            t[i][j] not in {-2, -1, 0, 1, 2},
            (s[i - 1][j], s[i + 1][j], s[i][j - 1], s[i][j + 1]) in {(0, 0, 1, 1), (1, 1, 0, 0)}
        ]


satisfy(
    # no ship on borders
    [(s[0][k] == 0, s[-1][k] == 0, s[k][0] == 0, s[k][-1] == 0) for k in range(n + 2)],

    # respecting the specified row tallies
    [Sum(s[i + 1]) == k for i, k in enumerate(rowSums)],

    # respecting the specified column tallies
    [Sum(s[:, j + 1]) == k for j, k in enumerate(colSums)],

    # being careful about cells on diagonals
    [(s[i][j], s[i - 1][j - 1], s[i - 1][j + 1], s[i + 1][j - 1], s[i + 1][j + 1]) in {(0, ANY, ANY, ANY, ANY), (1, 0, 0, 0, 0)}
     for i in range(1, n + 1) for j in range(1, n + 1)],

    # tag(channeling)
    [iff(s[i][j] == 1, t[i][j] != 0) for i in range(n + 2) for j in range(n + 2)],

    # counting the number of occurrences of ship segments of each type
    Cardinality(t[1:n + 1, 1:n + 1], occurrences={pos[i]: cp[i] for i in range(nTypes)} + {neg[i]: cn[i] for i in range(nTypes)}),

    # ensuring the right number of occurrences of ship segments of each type
    [cp[i] + cn[i] == surfaces[i] for i in range(nTypes)],

    # ensuring row connectedness of ship segments
    [t[i + 1] in horizontal_automaton for i in range(n)],

    # ensuring column connectedness of ship segments
    [t[:, j + 1] in vertical_automaton for j in range(n)],

    # tag(clues)
    [hint_ctr(c, i, j) for (c, i, j) in hints] if hints else None
)

# Note that:

# a) another way of setting border values to 0 is with constraints 'Sum', as in:
#   [Sum(s[0]) == 0, Sum(s[n + 1]) == 0, Sum(s[:, 0]) == 0, Sum(s[:, n + 1]) == 0],



"""
See Ian P. Gent, Christopher Jefferson, Ian Miguel: Watched Literals for Constraint Propagation in Minion. CP 2006: 182-197

Examples of Execution:
  python3 LangfordBin.py
  python3 LangfordBin.py -data=10
"""

from pycsp3 import *

n = data or 8

# v[i] is the ith value of the Langford's sequence
v = VarArray(size=2 * n, dom=range(1, n + 1))

# p[j] is the first (resp., second) position of 1+j/2 in v if j is even (resp., odd)
p = VarArray(size=2 * n, dom=range(2 * n))

satisfy(
    [v[p[2 * i]] == i + 1 for i in range(n)],

    [v[p[2 * i + 1]] == i + 1 for i in range(n)],

    [p[2 * i] == i + 2 + p[2 * i + 1] for i in range(n)]
)



from pycsp3.problems.data.parsing import *

n = next_int()  # number of variables
e = next_int()  # number rof constraints
next_line()

objective_coefficients = [next_int() for _ in range(n)]  # objective coefficients
coefficients = [[next_int() for _ in range(n)] for _ in range(e)]  # constraint coefficients
limits = [next_int() for _ in range(e)]  # constraint limits

assert e == len(coefficients) == len(limits)

data["coefficients"] = objective_coefficients
data["constraints"] = [OrderedDict([("coeffs", coefficients[i]), ("limit", limits[i])]) for i in range(e)]



"""
Problem 054 on CSPLib

Examples of Execution:
  python3 Queens.py
  python3 Queens.py -data=6
  python3 Queens.py -data=6 -variant=v1
  python3 Queens.py -data=6 -variant=v2
"""

from pycsp3 import *

n = data or 8

# q[i] is the column where is put the ith queen (at row i)
q = VarArray(size=n, dom=range(n))

if not variant():
    satisfy(
        AllDifferent(q),

        # controlling no two queens on the same upward diagonal
        AllDifferent(q[i] + i for i in range(n)),

        # controlling no two queens on the same downward diagonal
        AllDifferent(q[i] - i for i in range(n))
    )
elif variant("v1"):
    satisfy(
        AllDifferent(q),

        [abs(q[i] - q[j]) != abs(i - j) for i, j in combinations(n, 2)]
    )

elif variant("v2"):
    satisfy(
        (q[i] != q[j]) & (abs(q[i] - q[j]) != abs(i - j)) for i, j in combinations(n, 2)
    )



"""
See "Radio Link Frequency Assignment", by B. Cabon, S. de Givry, L. Lobjois, T. Schiex, J. Warners, Constraints An Int. J. 4(1): 79-89 (1999)

Examples of Execution:
  python3 Rlfap.py -data=Rlfap_card-scen-04.json -variant=card
  python3 Rlfap.py -data=Rlfap_span-scen-05.json -variant=span
  python3 Rlfap.py -data=Rlfap_max-graph-05.json -variant=max
"""

from pycsp3 import *

domains, variables, constraints, interferenceCosts, mobilityCosts = data
n = len(variables)

# f[i] is the frequency of the ith radio link
f = VarArray(size=n, dom=lambda i: domains[variables[i].domain])

satisfy(
    # managing pre-assigned frequencies
    [f[i] == v for i, (_, v, mob) in enumerate(variables) if v and not (variant("max") and mob)],

    # hard constraints on radio-links
    [expr(op, abs(f[i] - f[j]), k) for (i, j, op, k, wgt) in constraints if not (variant("max") and wgt)]
)

if variant("span"):
    minimize(
        # minimizing the largest frequency
        Maximum(f)
    )
elif variant("card"):
    minimize(
        # minimizing the number of used frequencies
        NValues(f)
    )
elif variant("max"):
    minimize(
        # minimizing the sum of violation costs
        Sum(ift(f[i] == v, 0, mobilityCosts[mob]) for (i, v, mob) in variables if v and mob)
        + Sum(ift(expr(op, abs(f[i] - f[j]), k), 0, interferenceCosts[wgt]) for (i, j, op, k, wgt) in constraints if wgt)
    )


# Note that:

# a) expr allows us to build an expression (constraint) with an operator given as first parameter (possibly, a string)
#    otherwise, it could have been written: abs(f[i] - f[j]) == k if op == "=" else abs(f[i] - f[j]) > k



from pycsp3.problems.data.parsing import *

nRestaurants = number_in(line())
next_line()
names = [next_line()[1:-2] for _ in range(nRestaurants)]
names[-1] = names[-1][:-1]  # because an additional symbol ']' on this line
next_line()
positions = [number_in(next_line()) for _ in range(nRestaurants)]

data["nDepots"] = number_in(next_line())
data["restaurants"] = [OrderedDict([("name", names[i]), ("position", positions[i])]) for i in range(nRestaurants)]



"""
See, e.g., "Fast Global Filtering for Eternity II" by Eric Bourreau and Thierry Benoist. Constraint Programming Letters, 2008, 3, pp.036-049

Example of Execution:
  python3 Eternity.py -data=Eternity_07x05.json
"""

from pycsp3 import *

n, m, pieces = data
assert n * m == len(pieces), "badly formed data"
max_value = max(max(piece) for piece in pieces)  # max possible value on pieces

table = {(i, piece[r % 4], piece[(r + 1) % 4], piece[(r + 2) % 4], piece[(r + 3) % 4]) for i, piece in enumerate(pieces) for r in range(4)}

# x[i][j] is the index of the piece at row i and column j
x = VarArray(size=[n, m], dom=range(n * m))

# t[i][j] is the value at the top of the piece at row i and column j
t = VarArray(size=[n + 1, m], dom=range(max_value + 1))

# l[i][j] is the value at the left of the piece at row i and column j
l = VarArray(size=[n, m + 1], dom=range(max_value + 1))

satisfy(
    # all pieces must be placed (only once)
    AllDifferent(x),

    # all pieces must be valid (i.e., must correspond to those given initially, possibly after applying some rotation)
    [(x[i][j], t[i][j], l[i][j + 1], t[i + 1][j], l[i][j]) in table for i in range(n) for j in range(m)],

    # putting special value 0 on borders
    [z == 0 for z in t[0] + l[:, -1] + t[-1] + l[:, 0]]
)



"""
See https://en.wikipedia.org/wiki/Travelling_salesman_problem

Examples of Execution:
  python3 TravelingSalesman.py -data=TravelingSalesman_10-20-0.json
  python3 TravelingSalesman.py -data=TravelingSalesman_10-20-0.json -variant=table
"""

from pycsp3 import *

distances = data
nCities = len(distances)

# c[i] is the ith city of the tour
c = VarArray(size=nCities, dom=range(nCities))

# d[i] is the distance between the cities i and i+1 chosen in the tour
d = VarArray(size=nCities, dom={v for row in distances for v in row})

satisfy(
    # Visiting each city only once
    AllDifferent(c)
)

if not variant():
    satisfy(
        # computing the distance between any two successive cities in the tour
        distances[c[i]][c[(i + 1) % nCities]] == d[i] for i in range(nCities)
    )

elif variant("table"):
    table = {(i, j, distances[i][j]) for i in range(nCities) for j in range(nCities) if i != j}

    satisfy(
        # computing the distance between any two successive cities in the tour
        (c[i], c[(i + 1) % nCities], d[i]) in table for i in range(nCities)
    )

minimize(
    # minimizing the travelled distance
    Sum(d)
)



import os
import shutil
import subprocess
import sys

from pycsp3.solvers.solver import class_path_abscon
from pycsp3.tools.utilities import BLUE, GREEN, ORANGE, RED, WHITE, WHITE_BOLD

COLOR_PY, COLOR_JV = BLUE, ORANGE

DATA_PATH = "pycsp3" + os.sep + "problems" + os.sep + "data" + os.sep
XCSP_PATH = "pycsp3" + os.sep + "problems" + os.sep + "tests" + os.sep + "xcsp" + os.sep


def run(xcsp, diff=None, same=None):
    if len(sys.argv) == 1 or sys.argv[1] == "-xcsp":
        xcsp.load(mode=2)
    elif sys.argv[1] == "-same":
        same.load()
    elif sys.argv[1] == "-diff":
        diff.load()


waiting = False


class Tester:
    @staticmethod
    def system_command(command, origin, target):
        assert command in {"mv", "cp"}
        print(os.getcwd())
        if os.name == 'nt':
            command = "move" if command == 'mv' else command
            command = "copy" if command == 'cp' else command
        print(command, origin, target)
        if not os.path.isfile(origin):
            print("error: do not found the file " + origin)
            exit(0)
        subprocess.call([command, origin, target], stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    @staticmethod
    def xml_indent(file):
        cmd = ["pycsp3" + os.sep + "libs" + os.sep + "xmlindent" + os.sep + "xmlindent", '-i', '2', '-w', file]
        print(cmd)
        out, error = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
        if error.decode('utf-8') != "":
            print("XmlIndent stderr : ")

    @staticmethod
    def execute_compiler(title, command):
        print(BLUE + "Command:" + WHITE, command)
        out, error = subprocess.Popen(command.split(), stdout=subprocess.PIPE, stderr=subprocess.PIPE).communicate()
        # print(title + " stdout:")
        print(out.decode('utf-8'))
        if error.decode('utf-8') != "":
            print(title + " stderr : ")
            print(error.decode('utf-8'))

    @staticmethod
    def diff_files(file1, file2, tmp):
        command = "diff " + file1 + " " + file2
        with open(tmp, "wb") as out:
            subprocess.Popen(command.split(), stdout=out, stderr=None).communicate()
        with open(tmp, "r") as out:
            lines = out.readlines()
        os.remove(tmp)
        return lines

    def __init__(self, name=None, *, dir_pbs_py=None, dir_pbs_jv=None, dir_tmp=None, dir_prs_py=None, dir_prs_jv=None):
        if name is None:
            assert dir_pbs_py and dir_pbs_jv and dir_tmp
        else:
            dir_pbs_py = "pycsp3" + os.sep + "problems" + os.sep + name + os.sep
            dir_pbs_jv = "problems." + name + "."
            dir_tmp = "pycsp3" + os.sep + "problems" + os.sep + "tests" + os.sep + "tmp" + os.sep + name
            dir_prs_py = "pycsp3" + os.sep + "problems" + os.sep + "data" + os.sep + "parsers" + os.sep
            dir_prs_jv = "problems.generators."
            self.dir_xcsp = XCSP_PATH + name + os.sep

        self.tmpDiff = dir_tmp + os.sep + "tmpDiff.txt"
        if not os.path.exists(dir_tmp):
            os.makedirs(dir_tmp)
        else:
            shutil.rmtree(dir_tmp)
            os.makedirs(dir_tmp)
        self.dir_xml_py = dir_tmp + os.sep + "PyCSP" + os.sep
        self.dir_xml_jv = dir_tmp + os.sep + "JvCSP" + os.sep
        os.makedirs(self.dir_xml_py)
        os.makedirs(self.dir_xml_jv)

        self.dir_pbs_py = dir_pbs_py  # for problems (models)
        self.dir_pbs_jv = dir_pbs_jv  # for problems (models)

        self.dir_prs_py = dir_prs_py  # for data parsers
        self.dir_prs_jv = dir_prs_jv  # for data parsers

        self.name_xml = None

        self.counters = {"total": 0, "diff": 0, "err": 0}

        self.instances = []

    def xml_name(self, model, data, variant, prs_py, prs_jv, nameXML):
        s = model + ("-" + variant if variant else "")
        if nameXML:
            s += nameXML + ".xml"
        else:
            if data:
                if prs_py is None and prs_jv is None:
                    if data.endswith(".json"):
                        s += "-" + data[:-5] + ".xml"
                    else:
                        if "," in data and "[" in data and "]" in data:
                            s += "-" + "-".join(data[1:-1].split(",")) + ".xml"
                        elif "[" in data and "]" in data:
                            s += "-" + data[1:-1] + ".xml"
                        else:
                            s += "-" + data + ".xml"
                else:
                    s += "-" + data.split(".")[0] + ".xml"
            else:
                s += ".xml"
        return s

    def xml_path_py(self):
        return self.dir_xml_py + self.name_xml

    def xml_path_jv(self):
        return self.dir_xml_jv + self.name_xml

    def xml_path_xcsp(self):
        return self.dir_xcsp + self.name_xml

    def data_py(self, data):
        return None if data is None else DATA_PATH + "json" + os.sep + data if data.endswith(".json") else data

    def data_jv(self, data):
        return None if data is None else DATA_PATH + "json" + os.sep + data if data.endswith(".json") else data

    def _command_py(self, model, data, variant, prs_py, options_py):
        cmd = "python3 " + self.dir_pbs_py + model + ".py"
        if self.data_py(data):
            cmd += " -data=" + ("" if prs_py is None else DATA_PATH + "raw" + os.sep) + self.data_py(data)
        if prs_py:
            cmd += " -dataparser=" + self.dir_prs_py + prs_py
        if variant:
            cmd += " -variant=" + variant
        if options_py:
            cmd += " " + options_py
        return cmd

    def _command_jv(self, model, data, variant, prs_jv, special, dataSpecial):
        print()
        cmd = "java -cp " + class_path_abscon() + (" AbsCon " if special else " org.xcsp.modeler.Compiler ")
        cmd += "problems.generators." + prs_jv if prs_jv else self.dir_pbs_jv + model
        if self.data_jv(data):
            if dataSpecial:
                cmd += " " + str(dataSpecial)
            cmd += " " + (DATA_PATH + "raw" + os.sep if prs_jv else " -data=") + self.data_jv(data)
        if special:
            cmd += " -ic=false -export=file"
        if variant:
            cmd += " -variant=" + variant
        return cmd

    def add(self, model, *, data=None, variant=None, prs_py=None, prs_jv=None, special=None, dataSpecial=None, nameXML=None, options_py=None):
        if isinstance(model, list):
            assert data == variant == prs_py == prs_jv == special == dataSpecial == nameXML == options_py is None
            for m in model:
                self.instances.append(m)
        else:
            self.instances.append((model, data, variant, prs_py, prs_jv, special, dataSpecial, nameXML, options_py))
        return self

    def load(self, *, mode=1):
        for model, data, variant, prs_py, prs_jv, special, dataSpecial, nameXML, options_py in self.instances:
            assert self.dir_pbs_py, "Call the __init__() function of g6_testing (constructor) before execute()"
            assert self.dir_pbs_jv, "Call the __init__() function of g6_testing (constructor) before execute()"
            assert self.dir_xml_py, "Call the __init__() function of g6_testing (constructor) before execute()"

            self.name_xml = self.xml_name(model, data, variant, prs_py, prs_jv, nameXML)  # name of the XML file that will be generated
            if os.path.isfile(self.xml_path_py()):
                os.remove(self.xml_path_py())
            if os.path.isfile(self.xml_path_jv()):
                os.remove(self.xml_path_jv())
            self.print_information(model, data, variant, prs_py, prs_jv)

            self.execute_compiler("PyCSP", self._command_py(model, data, variant, prs_py if not prs_py or prs_py[-1] == 'y' else prs_py + ".py", options_py))
            shutil.move(self.name_xml, self.xml_path_py())
            if mode == 1:  # comparison with jv
                self.execute_compiler("JvCSP", self._command_jv(model, data, variant, prs_jv, special, dataSpecial))
                if os.name != 'nt':
                    self.xml_indent(self.name_xml)
                shutil.move(self.name_xml, self.xml_path_jv())
                if os.name != 'nt':
                    self.check()
            elif mode == 2:  # comparison with recorded XCSP files
                if os.name != 'nt':
                    # shutil.copy(self.dir_xcsp + self.name_xml, self.xml_path_jv())  # we copy the xcsp file in the java dir to simualte a comparison with JvCSP
                    print("  Comparing PyCSP outcome with the XCSP3 file stored in " + self.dir_xcsp)
                    self.check(True)
            else:
                with open(self.xml_path_py(), "r") as f:
                    for line in f.readlines():
                        print(COLOR_PY + line[0:-1])
            if os.name != 'nt':
                os.system("rm -rf *.*~")  # for removing the temporary files

    def check(self, xcsp=False):
        f = self.xml_path_xcsp() if xcsp else self.xml_path_jv()
        self.counters["total"] += 1
        if not os.path.isfile(self.xml_path_py()) or not os.path.isfile(f):
            print("error: files not found " + self.xml_path_py() + " or " + f)
            self.counters["err"] += 1
            if waiting:
                input("Press Enter to continue...")
        else:
            lines = self.diff_files(self.xml_path_py(), f, self.tmpDiff)
            if len(lines) == 0:
                print("  => No difference for " + self.name_xml)
            else:
                print("  => Several differences (" + str(len(lines)) + ") in " + self.name_xml)
                self.counters["diff"] += 1
                self.print_differences(lines, limit=20 if len(lines) > 200 else None, xcsp=xcsp)
                if waiting:
                    input("Press Enter to continue...")
        diff = (RED if self.counters["diff"] > 0 else GREEN) + str(self.counters["diff"])
        err = (RED if self.counters["err"] > 0 else GREEN) + str(self.counters["err"])
        print("\n" + WHITE_BOLD + "[Currently] " + diff + WHITE + " difference(s) on " + str(
            self.counters["total"]) + " test(s) (" + err + WHITE + " error(s))\n")

    def print_differences(self, lines, limit, xcsp=False):
        print(COLOR_PY + "PyCSP" + WHITE + " vs. " + COLOR_JV + ("JvCSP" if not xcsp else "XCSP") + WHITE + " differences:\n")
        if limit is None:
            for line in lines:
                if line[0] in {'>', '<'}:
                    print(COLOR_JV + line[2:-1] if line[0] == '>' else COLOR_PY + line[2:-1])
        else:
            cnt = 0
            for line in lines:
                if line[0] == '>':
                    print(COLOR_JV + line[2:-1])
                    cnt += 1
                    if cnt > limit:
                        break
            cnt = 0
            for line in lines:
                if line[0] == '<':
                    print(COLOR_PY + line[2:-1])
                    cnt += 1
                    if cnt > limit:
                        break
            print(WHITE + "Too many differences (" + str(len(lines)) + ") in " + self.name_xml + ": only " + str(limit) + " displayed lines")
        print(WHITE)

    def print_information(self, model, data, variant, prs_py, prs_jv):
        print("\n" + RED + "|================================================================|" + WHITE)
        print("  Name: " + model + ("    Variant: " + variant if variant else ""))
        if data:
            print("  Data: " + data)
        print("  Name XML: " + self.name_xml)
        if prs_py:
            print("  parser py: " + prs_py)
        if prs_jv:
            print("  parser jv: " + prs_jv)
        print(RED + "|================================================================|" + WHITE)



"""
See https://en.wikipedia.org/wiki/Light_Up_(puzzle)

Examples of Execution:
  python3 Lightup.py -data=Lightup_example.json
  python3 Lightup.py -data=Lightup_example.txt -dataparser=Lightup_Parser.py
"""

from pycsp3 import *

grid = data  # -1 is for a white cell, 0 to 4 for a hint and 5 for a black cell
n, m = len(grid), len(grid[0])


def scope(i, j, mode):
    scp = []
    if mode == 'h':  # horizontal scope
        if j == 0 or grid[i][j - 1] != -1 and grid[i][j] == -1:  # a left border
            while j < m and grid[i][j] == -1:
                scp.append(x[i][j])
                j += 1
    elif mode == 'v':  # vertical scope
        if i == 0 or grid[i - 1][j] != -1 and grid[i][j] == -1:  # a top border
            while i < n and grid[i][j] == -1:
                scp.append(x[i][j])
                i += 1
    elif mode == 'f':  # full scope
        if grid[i][j] != -1:
            return None
        base = j
        while base >= 0 and grid[i][base] == -1:
            base -= 1
        for jj in range(base + 1, m):
            if grid[i][jj] != -1:
                break
            scp.append(x[i][jj])
        base = i
        while base >= 0 and grid[base][j] == -1:
            base -= 1
        for ii in range(base + 1, n):
            if grid[ii][j] != -1:
                break
            if ii != i:
                scp.append(x[ii][j])
    elif mode == 's':  # side scope
        if i > 0 and grid[i - 1][j] == -1:
            scp.append(x[i - 1][j])
        if i < n - 1 and grid[i + 1][j] == -1:
            scp.append(x[i + 1][j])
        if j > 0 and grid[i][j - 1] == -1:
            scp.append(x[i][j - 1])
        if j < m - 1 and grid[i][j + 1] == -1:
            scp.append(x[i][j + 1])
    limit = 2 if mode in {'h', 'v'} else 1
    return scp if len(scp) >= limit else None


# x[i][j] is 1 iff a light bulb is put at row i and col j
x = VarArray(size=[n, m], dom={0, 1})

satisfy(
    # at most 1 bulb on each maximal sequence of white cells on rows
    [Count(scp) <= 1 for scp in [scope(i, j, 'h') for i in range(n) for j in range(m)] if scp],

    # at most 1 bulb on each maximal sequence of white cells on columns
    [Count(scp) <= 1 for scp in [scope(i, j, 'v') for i in range(n) for j in range(m)] if scp],

    [Count(scp) >= 1 for scp in [scope(i, j, 'f') for i in range(n) for j in range(m)] if scp],

    # tag(clues)
    [Count(scp) == k for (scp, k) in [(scope(i, j, 's'), grid[i][j]) for i in range(n) for j in range(m)] if scp and 0 <= k <= 4]
)



"""
This is a Radar surveillance instance where some radars must be put on a geographic area of a specified size and must cover all cells.
There are some insignificant cells that must not be covered by any radar. All other cells must be covered by exactly a specified number of radars.
Instances of this problem follow the description given by the Swedish Institute of Computer Science (SICS).

Example of Execution:
  python3 RadarSurveillance.py -data=RadarSurveillance_8-24-3-2-00.json
"""

from enum import Enum

from pycsp3 import *

mapSize, maxCoverage, radars, insignificantCells = data
nRadars = len(radars)


class Sector(Enum):
    NORTH_EAST, EAST, SOUTH_EAST, SOUTH_WEST, WEST, NORTH_WEST = range(6)

    def row_right_cell(self, i):
        return i + (-1 if self in {Sector.NORTH_EAST, Sector.NORTH_WEST} else 0 if self in {Sector.EAST, Sector.WEST} else 1)

    def row_left_cell(self, i):
        return i + (-1 if self in {Sector.NORTH_EAST, Sector.EAST} else 0 if self in {Sector.SOUTH_EAST, Sector.NORTH_WEST} else 1)

    def col_right_cell(self, i, j):
        ro = 0 if i % 2 == 1 else 1
        return j + (1 if self == Sector.EAST else -1 if self == Sector.WEST else ro if self in {Sector.NORTH_EAST, Sector.SOUTH_EAST} else ro - 1)

    def col_left_cell(self, i, j):
        ro = 0 if i % 2 == 1 else 1
        return j + (1 if self == Sector.SOUTH_EAST else -1 if self == Sector.NORTH_WEST else ro if self in {Sector.EAST, Sector.SOUTH_WEST} else ro - 1)

    def distance(self, i, j, k, l, curr_distance):
        if curr_distance > maxCoverage:
            return -1
        if (i, j) == (k, l):
            return curr_distance
        d = self.distance(i, j, self.row_right_cell(k), self.col_right_cell(k, l), curr_distance + 1)
        return d if d != -1 else self.distance(i, j, self.row_left_cell(k), self.col_left_cell(k, l), curr_distance + 1)


def cell_constraint(i, j):
    scope, dists = [], []
    for ir, radar in enumerate(radars):
        for sector in Sector:
            d = sector.distance(i, j, sector.row_right_cell(radar[0]), sector.col_right_cell(radar[0], radar[1]), 1)
            if d != -1:
                scope.append(x[ir][sector.value])
                dists.append(d)
    arity = len(scope)
    if arity == 0:
        return None
    significant_cell = [i, j] not in insignificantCells
    if arity == 1:
        return scope[0] > dists[0] - 1 if significant_cell else scope[0] < dists[0]
    return Sum(scope[i] >= dists[i] for i in range(arity)) == (min(arity, maxCoverage) if significant_cell else 0)


# x[i][j] is the power of the ith radar in the jth sector
x = VarArray(size=[nRadars, len(Sector)], dom=range(maxCoverage + 1))

satisfy(
    cell_constraint(i, j) for i in range(mapSize) for j in range(mapSize)
)



"""
See https://en.wikipedia.org/wiki/Nonogram

Examples of Execution:
  python3 Nonogram.py -data=Nonogram_example.json
  python3 Nonogram.py -data=Nonogram_example.json -variant=table
  python3 Nonogram.py -data=Nonogram_example.txt -dataparser=Nonogram_Parser.py
"""

from pycsp3 import *

rows, cols = data  # patterns for row and columns
nRows, nCols = len(rows), len(cols)

#  x[i][j] is 1 iff the cell at row i and col j is colored in black
x = VarArray(size=[nRows, nCols], dom={0, 1})

if not variant():
    def automaton(pattern):
        q = Automaton.q  # for building state names
        transitions = []
        if len(pattern) == 0:
            n_states = 1
            transitions.append((q(0), 0, q(0)))
        else:
            n_states = sum(pattern) + len(pattern)
            num = 0
            for i in range(len(pattern)):
                transitions.append((q(num), 0, q(num)))
                for j in range(pattern[i]):
                    transitions.append((q(num), 1, q(num + 1)))
                    num += 1
                if i < len(pattern) - 1:
                    transitions.append((q(num), 0, q(num + 1)))
                    num += 1
            transitions.append((q(num), 0, q(num)))
        return Automaton(start=q(0), final=q(n_states - 1), transitions=transitions)


    satisfy(
        [x[i] in automaton(rows[i]) for i in range(nRows)],

        [x[:, j] in automaton(cols[j]) for j in range(nCols)]
    )

elif variant("table"):
    cache = dict()


    def table(pattern, row):
        def build_from(lst, tmp, i, k):
            s = sum([pattern[e] for e in range(k, len(pattern))])
            if i + s + (len(pattern) - 1 - k) > len(tmp):
                return lst
            if i == len(tmp):
                lst.append(tuple(tmp))
            else:
                tmp[i] = 0
                build_from(lst, tmp, i + 1, k)
                if k < len(pattern):
                    for j in range(i, i + pattern[k]):
                        tmp[j] = 1
                    if i + pattern[k] == len(tmp):
                        build_from(lst, tmp, i + pattern[k], k + 1)
                    else:
                        tmp[i + pattern[k]] = 0
                        build_from(lst, tmp, i + pattern[k] + 1, k + 1)
            return lst

        key = str("R" if row else "C") + "".join(str(pattern))
        if key not in cache:
            cache[key] = build_from([], [0] * (nCols if row else nRows), 0, 0)
        return cache[key]


    satisfy(
        [x[i] in table(rows[i], row=True) for i in range(nRows)],

        [x[:, j] in table(cols[j], row=False) for j in range(nCols)]
    )



from enum import Enum, unique


def auto(n_occurrences=1):
    def _auto():  # To be replaced by auto() in python 3.6 ?
        if not hasattr(auto, "cnt"):
            auto.cnt = 0
        auto.cnt += 1
        return auto.cnt

    return _auto() if n_occurrences == 1 else (_auto() for _ in range(n_occurrences))


@unique
class TypeFramework(Enum):
    CSP, COP = auto(2)

    def __str__(self):
        return self.name


class AbstractType(Enum):
    def __str__(self):
        if hasattr(self, 'ccname'):  # camel case name
            return self.ccname
        t = self.name.split("_")
        self.ccname = t[0].lower() + ''.join(t[i][0] + t[i][1:].lower() for i in range(1, len(t)))
        return self.ccname


@unique
class TypeVar(AbstractType):
    INTEGER, SYMBOLIC, REAL = auto(3)
    STOCHASTIC, SYMBOLIC_STOCHASTIC = auto(2)
    SET, SYMBOLIC_SET = auto(2)
    UNDIRECTED_GRAPH, DIRECTED_GRAPH = auto(2)
    POINT, INTERVAL, REGION = auto(3)

    def is_basic(self):
        return self in {TypeVar.INTEGER, TypeVar.SYMBOLIC, TypeVar.REAL} or self.is_stochastic()

    def is_stochastic(self):
        return self in {TypeVar.STOCHASTIC, TypeVar.SYMBOLIC_STOCHASTIC}

    def is_set(self):
        return self in {TypeVar.SET, TypeVar.SYMBOLIC_SET}

    def is_graph(self):
        return self in {TypeVar.UNDIRECTED_GRAPH, TypeVar.DIRECTED_GRAPH}

    def is_qualitative(self):
        return self in {TypeVar.POINT, TypeVar.INTERVAL, TypeVar.REGION}


@unique
class TypeCtr(AbstractType):
    EXTENSION, INTENSION, SMART = auto(3)
    REGULAR, GRAMMAR, MDD = auto(3)
    ALL_DIFFERENT, ALL_EQUAL, ALL_DISTANT, ORDERED, LEX, ALL_INTERSECTING = auto(6)
    SUM, COUNT, N_VALUES, CARDINALITY, BALANCE, SPREAD, DEVIATION, SUM_COSTS, RANGE, ROOTS = auto(10)
    MINIMUM, MAXIMUM, ELEMENT, CHANNEL, PERMUTATION, PRECEDENCE, PARTITION = auto(7)
    STRETCH, NO_OVERLAP, CUMULATIVE, BIN_PACKING, KNAPSACK, NETWORK_FLOW = auto(6)
    CIRCUIT, N_CIRCUITS, PATH, N_PATHS, TREE, N_TREES, ARBO, N_ARBOS, N_CLIQUES = auto(9)
    CLAUSE, INSTANTIATION = auto(2)
    AND, OR, NOT, IFF, IF_THEN, IF_THEN_ELSE = auto(6)
    SLIDE, SEQBIN = auto(2)
    MINIMIZE, MAXIMIZE = auto(2)  # used for posting objectives (not exactly constraints)


@unique
class TypeCtrArg(AbstractType):
    LIST, SET, MSET, MATRIX = auto(4)
    FUNCTION, SUPPORTS, CONFLICTS = auto(3)
    EXCEPT, VALUE, VALUES, TOTAL, COEFFS, CONDITION = auto(6)
    COST, OPERATOR, NUMBER = auto(3)
    TRANSITIONS, START, FINAL, TERMINAL, RULES = auto(5)
    INDEX = auto()
    MAPPING = auto()
    OCCURS, ROW_OCCURS, COL_OCCURS = auto(3)
    WIDTHS, PATTERNS = auto(2)
    ORIGINS, LENGTHS, ENDS, HEIGHTS, MACHINES, CONDITIONS = auto(6)
    SIZES, WEIGTHS, PROFITS = auto(3)
    SIZE, ROOT, IMAGE, GRAPH, ROW, EXPRESSION, TYPE = auto(7)
    START_INDEX, START_ROW_INDEX, START_COL_INDEX = auto(3)


@unique
class TypeXML(AbstractType):
    FORMAT, TYPE, ID, CLASS = auto(4)
    NOTE, AS, SIZE = auto(3)
    VIOLATION_MEASURE, VIOLATION_PARAMETERS, DEFAULT_COST, VIOLATION_COST, COST = auto(5)
    REIFIED_BY, HREIFIED_FROM, HREIFIED_TO = auto(3)
    CLOSED, FOR, RESTRICTION, RANK = auto(4)
    START_INDEX, ZERO_IGNORED, CASE, ORDER, CIRCULAR, OFFSET, COLLECT, VIOLABLE = auto(8)
    LB, UB, COMBINATION = auto(3)
    INSTANCE = auto()
    VARIABLES, VAR, ARRAY, DOMAIN, REQUIRED, POSSIBLE = auto(6)
    CONSTRAINTS, BLOCK, GROUP, ARGS = auto(4)
    OBJECTIVES, OBJECTIVE, MINIMIZE, MAXIMIZE = auto(4)
    SOFT = auto()
    ANNOTATIONS, DECISION, OUTPUT, VAR_HEURISTIC, VAL_HEURISTIC, PREPRO, SEARCH, RESTARTS, FILTERING = auto(9)


@unique
class TypeConditionOperator(AbstractType):
    LT, LE, GE, GT, EQ, NE, IN, NOTIN = auto(8)

    @staticmethod
    def value_of(s):
        assert isinstance(s, str)
        try:
            return TypeConditionOperator[s.upper()]
        except KeyError:
            if s == "<":
                return TypeConditionOperator.LT
            if s == "<=":
                return TypeConditionOperator.LE
            if s == ">=":
                return TypeConditionOperator.GE
            if s == ">":
                return TypeConditionOperator.GT
            if s in {"=", "=="}:
                return TypeConditionOperator.EQ
            if s == "!=":
                return TypeConditionOperator.NE
        raise ValueError


@unique
class TypeOrderedOperator(Enum):
    STRICTLY_INCREASING, INCREASING, DECREASING, STRICTLY_DECREASING = auto(4)

    def __str__(self):
        if self == TypeOrderedOperator.STRICTLY_INCREASING:
            return "lt"
        if self == TypeOrderedOperator.INCREASING:
            return "le"
        if self == TypeOrderedOperator.DECREASING:
            return "ge"
        assert self == TypeOrderedOperator.STRICTLY_DECREASING
        return "gt"


@unique
class TypeRank(Enum):
    FIRST, LAST, ANY = auto(3)



"""
See Practice Problem for Google Hash Code 2017
See https://www.academia.edu/31537057/Pizza_Practice_Problem_for_Hash_Code_2017

Examples of Execution:
  python3 HCPizza.py -data=HCPizza_tiny.json
  python3 HCPizza.py -data=HCPizza_tiny.txt -dataparser=HCPizza_Parser.py
"""

from pycsp3 import *

minIngredients, maxSize, pizza = data
nRows, nCols = len(pizza), len(pizza[0])
patterns = [(i, j) for i in range(1, min(maxSize, nRows) + 1) for j in range(1, min(maxSize, nCols) + 1) if 2 * minIngredients <= i * j <= maxSize]
nPatterns = len(patterns)


def possible_slices():
    _overlaps = [[[] for _ in range(nCols)] for _ in range(nRows)]
    _possibleSlices = [[[False for _ in range(nPatterns)] for _ in range(nCols)] for _ in range(nRows)]
    for i, j, k in product(range(nRows), range(nCols), range(nPatterns)):
        height, width = patterns[k][0], patterns[k][1]
        n_mushrooms, n_tomatoes = 0, 0
        for ib, jb in product(range(i, min(i + height, nRows)), range(j, min(j + width, nCols))):
            if pizza[ib][jb] == 0:
                n_mushrooms += 1
            else:
                n_tomatoes += 1
        if n_mushrooms >= data.minIngredients and n_tomatoes >= data.minIngredients:
            _possibleSlices[i][j][k] = True
            for ib, jb in product(range(i, min(i + height, nRows)), range(j, min(j + width, nCols))):
                _overlaps[ib][jb].append((i, j, k))
    return _overlaps, _possibleSlices


overlaps, slices = possible_slices()


def pattern_size(i, j, k):
    return (min(i + patterns[k][0], nRows) - i) * (min(j + patterns[k][1], nCols) - j)


# x[i][j][k] is 1 iff the slice with left top cell at (i,j) and pattern k is selected
x = VarArray(size=[nRows, nCols, nPatterns], dom=lambda i, j, k: {0, 1} if slices[i][j][k] else None)

# s[i][j][k] is the size of the slice with left top cell at (i,j) and pattern k (0 if the slice is not selected)
s = VarArray(size=[nRows, nCols, nPatterns], dom=lambda i, j, k: {0, pattern_size(i, j, k)} if slices[i][j][k] else None)

satisfy(
    # computing sizes of selected slices
    [(x[i][j][k], s[i][j][k]) in {(0, 0), (1, pattern_size(i, j, k))} for i, j, k in product(range(nRows), range(nCols), range(nPatterns)) if slices[i][j][k]],

    # ensuring that no two slices overlap
    [Sum([x[t[0]][t[1]][t[2]] for t in overlaps[i][j]]) <= 1 for i in range(nRows) for j in range(nCols) if len(overlaps[i][j]) > 1]
)

maximize(
    # maximizing the number of selected pizza cells
    Sum(s)
)



"""
See https://en.wikipedia.org/wiki/Knight%27s_tour

Examples of Execution:
  python3 KnightTour.py
  python3 KnightTour.py -data=16
  python3 KnightTour.py -data=16 -variant=table-2
  python3 KnightTour.py -data=16 -variant=table-3
"""

from pycsp3 import *

n = data or 8

# x[i] is the cell number where the ith knight is put
x = VarArray(size=n * n, dom=range(n * n))

satisfy(
    AllDifferent(x)
)

if not variant():
    satisfy(
        knight_attack(x[i], x[(i + 1) % (n * n)], n) for i in range(n * n)
    )

elif variant("table"):
    r = int(subvariant())
    assert r > 1 and n % (r - 1) == 0


    def jump(i, j):
        t = []
        if i - 2 >= 0:
            if j - 1 >= 0:
                t.append((i - 2) * n + j - 1)
            if j + 1 < n:
                t.append((i - 2) * n + j + 1)
        if i + 2 < n:
            if j - 1 >= 0:
                t.append((i + 2) * n + j - 1)
            if j + 1 < n:
                t.append((i + 2) * n + j + 1)
        if j - 2 >= 0:
            if i - 1 >= 0:
                t.append((i - 1) * n + j - 2)
            if i + 1 < n:
                t.append((i + 1) * n + j - 2)
        if j + 2 < n:
            if i - 1 >= 0:
                t.append((i - 1) * n + j + 2)
            if i + 1 < n:
                t.append((i + 1) * n + j + 2)
        return sorted(t)


    jumps = [jump(i, j) for i in range(n) for j in range(n)]


    def table_recursive(i, tmp):
        if i == len(tmp):
            table.add(tuple(tmp[:]))
        else:
            for v in jumps[tmp[i - 1]]:
                if len([j for j in range(0, i - 1) if tmp[j] == v]) == 0:
                    tmp[i] = v
                    table_recursive(i + 1, tmp)


    table = set()
    for i in range(n * n):
        table_recursive(1, [i] + [0] * (r - 1))

    satisfy(
        [x[(i + j) % (n * n)] for j in range(r)] in table for i in range(0, n * n, r - 1)
    )

satisfy(
    # breaking symmetries by putting the first knight in the first cell, and the second knight in the first possible cell  tag(symmetry-breaking)
    [x[0] == 0, x[1] == n + 2]
)



from pycsp3 import *

n, k = data  # n is the order(board width)  -- k is the number of knights

#  x[i] is the cell number of the board where is put the ith knight
x = VarArray(size=k, dom=range(n * n))

satisfy(
    [x[i] != x[j] for i in range(k) for j in range(i + 2, k) if i != 0 or j != k - 1],

    [knight_attack(x[i], x[(i + 1) % k], n) for i in range(k)]
)



"""
See "The First Evaluation of Pseudo-Boolean Solvers (PB'05)" by Vasco M. Manquinho and Olivier Roussel. J. Satisf. Boolean Model. Comput. 2(1-4): 103-143 (2006)

Example of Execution:
  python3 PseudoBoolean.py -data=PseudoBoolean_example.opb -dataparser=PseudoBoolean_Parser.py
"""

from pycsp3 import *

n, e, constraints, objective = data  # n and e respectively denote the numbers of variables and constraints

x = VarArray(size=n, dom={0, 1})

satisfy(
    # respecting each linear constraint
    Sum([x[i] for i in nums] * coeffs, condition=(op, limit)) for (coeffs, nums, op, limit) in constraints
)

if objective:
    minimize(
        # minimizing the linear objective
        [x[i] for i in objective.nums] * objective.coeffs
    )



from pycsp3.problems.data.parsing import *

n = number_in(line())
data["weights"] = [numbers_in(next_line()) for _ in range(n)]  # empty lines are discarced by the dataparser
data["distances"] = [numbers_in(next_line()) for _ in range(n)]



"""
Problem 081 on CSPLib

Example of Execution:
  python3 Blackhole.py -data=Blackhole-01.json
"""

from pycsp3 import *

m, piles = data  # m denotes the number of cards per suit
nCards = 4 * m

# x[i] is the value j of the card at the ith position of the built stack
x = VarArray(size=nCards, dom=range(nCards))

# y[j] is the position i of the card whose value is j
y = VarArray(size=nCards, dom=range(nCards))

table = {(i, j) for i in range(nCards) for j in range(nCards) if i % m == (j + 1) % m or j % m == (i + 1) % m}

satisfy(
    # linking variables of x and y
    Channel(x, y),

    # the Ace of Spades is initially put on the stack
    y[0] == 0,

    # cards must be played in the order of the piles
    [Increasing([y[j] for j in pile], strict=True) for pile in piles],

    # each new card put on the stack must be at a rank higher or lower than the previous one
    [(x[i], x[i + 1]) in table for i in range(nCards - 1)]
)



"""
Problem 028 on CSPLib

Examples of Execution:
  python3 Bibd.py
  python3 Bibd.py -data=[9,0,0,3,9]
  python3 Bibd.py -data=[9,0,0,3,9] -variant=aux
"""

from pycsp3 import *

v, b, r, k, l = data or (6, 0, 0, 3, 8)
b = (l * v * (v - 1)) // (k * (k - 1)) if b == 0 else b  # when specified at 0, b is automatically computed
r = (l * (v - 1)) // (k - 1) if r == 0 else r  # when specified at 0, r is automatically computed

# x[i][j] is the value of the matrix at row i and column j
x = VarArray(size=[v, b], dom={0, 1})

if not variant():
    satisfy(
        # constraints on rows
        [Sum(row) == r for row in x],

        # constraints on columns
        [Sum(col) == k for col in columns(x)],

        # scalar constraints with respect to lambda
        [row1 * row2 == l for row1, row2 in combinations(x, 2)]
    )

elif variant("aux"):
    #  s[i][j][k] is the product of x[i][k] and x[j][k]
    s = VarArray(size=[v, v, b], dom={0, 1})

    satisfy(
        # constraints on rows
        [Sum(x[i]) == r for i in range(v)],

        # constraints on columns
        [Sum(x[:, j]) == k for j in range(b)],

        # computing scalar variables
        [s[i][j][k] == x[i][k] * x[j][k] for i, j in combinations(range(v), 2) for k in range(b)],

        # scalar constraints with respect to lambda
        [Sum(s[i][j]) == l for i, j in combinations(range(v), 2)]
    )

satisfy(
    # Increasingly ordering both rows and columns  tag(symmetry-breaking)
    LexIncreasing(x, matrix=True)
)



"""
Problem 010 on CSPLib

Examples of Execution:
  python3 SocialGolfers.py
  python3 SocialGolfers.py -data=[5,3,6]
  python3 SocialGolfers.py -data=[5,3,6] -variant=01
"""

from pycsp3 import *

nGroups, size, nWeeks = data or (4, 4, 5)  # size is the size of the groups
nPlayers = nGroups * size

if not variant():
    # g[w][p] is the group admitting on week w the player p
    g = VarArray(size=[nWeeks, nPlayers], dom=range(nGroups))

    satisfy(
        # ensuring that two players don't meet more than one time
        [(g[w1][p1] != g[w1][p2]) | (g[w2][p1] != g[w2][p2]) for w1, w2 in combinations(nWeeks, 2) for p1, p2 in combinations(nPlayers, 2)],

        # respecting the size of the groups
        [Cardinality(g[w], occurrences={i: size for i in range(nGroups)}) for w in range(nWeeks)],

        # tag(symmetry-breaking)
        [
            LexIncreasing(g, matrix=True),

            [g[0][p] == p // size for p in range(nPlayers)],

            [g[w][k] == k for k in range(size) for w in range(1, nWeeks)]
        ]
    )
elif variant("01"):
    # x[w][g][p] is 1 iff on week w the group g admits the player p
    x = VarArray(size=[nWeeks, nGroups, nPlayers], dom={0, 1})

    # tw[p1][p2][w] is 1 iff players p1 and p2 play together on week w
    tw = VarArray(size=[nPlayers, nPlayers, nWeeks], dom=lambda p1, p2, w: {0, 1} if p1 < p2 else None)

    # twg[p1][p2][w][g] is 1 iff players p1 and p2 play together on week w in group g
    twg = VarArray(size=[nPlayers, nPlayers, nWeeks, nGroups], dom=lambda p1, p2, w, g: {0, 1} if p1 < p2 else None)

    satisfy(
        # each week, each player plays exactly once
        [Sum(x[w][g][p] for g in range(nGroups)) == 1 for w in range(nWeeks) for p in range(nPlayers)],

        # each week, each group contains exactly the right number of players
        [Sum(x[w][g]) == size for w in range(nWeeks) for g in range(nGroups)],

        # two players cannot meet twice
        [Sum(tw[p1][p2]) <= 1 for p1, p2 in combinations(nPlayers, 2)],

        # deciding when two players play together
        [Sum(twg[p1][p2][w]) == tw[p1][p2][w] for p1, p2 in combinations(nPlayers, 2) for w in range(nWeeks)],

        # deciding when two players play in the same group
        [x[w][g][p1] * x[w][g][p2] == twg[p1][p2][w][g] for w in range(nWeeks) for g in range(nGroups) for p1, p2 in combinations(nPlayers, 2)],

        # tag(symmetry-breaking)
        [
            # each week, groups are strictly ordered
            [LexDecreasing(x[w], strict=True) for w in range(nWeeks)],

            # weeks are strictly ordered (it suffices to consider the first group)
            LexDecreasing([x[w][0] for w in range(nWeeks)], strict=True),

            # golfers are strictly ordered
            LexDecreasing([[x[w][g][p] for w in range(nWeeks) for g in range(nGroups)] for p in range(nPlayers)], strict=True)
        ]

    )



from pycsp3.problems.tests.tester import Tester, run

NAME = "g7_todo"

diff = (Tester(NAME)
        )

same = (Tester(NAME)
        .add("Molnar", data="[3,5]")
        )

xcsp = (Tester(NAME)
        .add(same.instances)
        )

run(xcsp, diff, same)



import atexit
import os, sys
#from collections import OrderedDict

__python_version__ = str(sys.version).split(os.linesep)[0].split(' ')[0]

if sys.version_info[0] < 3 or (sys.version_info[1] < 5 or sys.version_info[1] > 7):
    __message_version__ = os.linesep + " Must be using between Python 3.5 and Python 3.7" + os.linesep \
                    + " Your version is Python " + __python_version__  + os.linesep \
                    + " The version of pycsp3 made compatible with Python 3.8 will be released soon."
    raise Exception(__message_version__)

from pycsp3.functions import *  # keep it at first position (before Compilation)
from pycsp3.compiler import Compilation

__version__ = open(os.path.join(os.path.dirname(__file__), 'version.txt'), encoding='utf-8').read()

if sys.argv:
    if len(sys.argv) == 1 and sys.argv[0] == "-m":
        import shutil
        import pycsp3
        print("Python version: ", __python_version__)
        print("pycsp3 version: ", __version__)
        problems = os.sep.join(pycsp3.__file__.split(os.sep)[:-1]) + os.sep + "problems" + os.sep
        target = os.getcwd() + os.sep + "problems" + os.sep
        print("Source of files found: ", problems)
        shutil.copytree(problems, target, ignore=shutil.ignore_patterns('g6_testing', 'g7_todo', 'tests', '__init__.py', '__pycache__*'))
        print("Successfully created the directory " + target + " containing the problems !")
        exit(0)
    elif sys.argv[0] == '':
        Compilation.load(console=True)
        data = None
    else:
        # TODO: analysing if we have to compile (e..g, when running the tester, we should not try to do that)
        Compilation.load()
        data = Compilation.data


def compile():
    return Compilation.compile()


@atexit.register
def end():
    if not Compilation.done:
        Compilation.compile()



from pycsp3 import *

x = VarArray(size=[10, 4], dom=range(10))
y = VarArray(size=[4], dom=range(10))

satisfy(
    [Maximum(x[i]) == y[i] if i % 2 == 0 else Maximum(x[i]) > 4 for i in range(4)],
    [NValues(x[i]) == y[i] for i in range(4)],
    [Sum(x[:, i]) - y[i] < 3 for i in range(4)],  # TODO Sum(x[:, i]) < y[i] + 3  ... is not working
    [Maximum(x[i]) - Minimum(x[i]) < 6 for i in range(4)],
    [Maximum(x[i]) + Minimum(x[i]) < 6 for i in range(4)]
)



import json

from pycsp3.compiler import Compilation
from pycsp3.problems.data.parsing import *

# illustrating how to convert from a JSON format to another one.
# For example python3 PizzaVoucher.py -data=zn06-oldformat.json -dataparser=PizzaVoucher_Converter.py -dataexport


name = options.data  # the name of the JSON file whose format must be converted")
with open(name) as f:
    d = json.load(f)

pp, fp = d.get("voucherPayPart"), d.get("voucherFreePart")
assert len(pp) == len(fp)

data["pizzaPrices"] = d.get("pizzaPrices")
data["vouchers"] = [OrderedDict([("payPart", pp[i]), ("freePart", fp[i])]) for i in range(len(pp))]

pos = name.rfind("/")
Compilation.string_data = "-" + name[pos + 1:] if pos != -1 else name



from collections import OrderedDict

from lxml import etree

from pycsp3.classes.auxiliary.ptypes import TypeFramework, TypeXML, TypeVar, TypeCtr, TypeCtrArg
from pycsp3.classes.entities import Entity, EVar, EVarArray, ECtr, EObjective, EAnnotation, EGroup, \
    EBlock, ESlide, EIfThenElse, EToGather, EToSatisfy, CtrEntities, VarEntities, ObjEntities, AnnEntities
from pycsp3.classes.main.constraints import ConstraintIntension
from pycsp3.dashboard import options
from pycsp3.tools.compactor import compact
from pycsp3.tools.slider import _identify_slide

SIZE_LIMIT_FOR_USING_AS = 12  # when building domains of variables of arrays of variables (and using the attribute 'as')


class DefaultListOrderedDict(OrderedDict):
    def __missing__(self, k):
        self[k] = []
        return self[k]


def _text(elt, s):
    elt.text = " " + (" ".join(str(v) for v in s) if isinstance(s, (list, tuple)) else str(s)) + " "


def _element(name, entity=None, *, id=None, attributes=[], text=None):
    elt = etree.Element(str(name))
    if id is not None:
        assert isinstance(id, str)
        elt.set(str(TypeXML.ID), id)
    if entity is not None:
        assert isinstance(entity, Entity)
        if len(entity.tags) > 0:
            elt.set(str(TypeXML.CLASS), ' '.join(tag for tag in entity.tags))
        if entity.comment and options.nocomments is False:
            elt.set(str(TypeXML.NOTE), entity.comment)
    attributes = attributes if isinstance(attributes, list) else [attributes]
    for (k, v) in attributes:
        elt.set(str(k), str(v))
    if text is not None:
        _text(elt, text)
    return elt


def _complex_var(va, dom_or_dom2vars):
    elt = _element(TypeXML.VAR if isinstance(va, EVar) else TypeXML.ARRAY, va, id=va.id)
    if isinstance(va, EVarArray):
        elt.set(str(TypeXML.SIZE), va.size_to_string())
    if not isinstance(dom_or_dom2vars, DefaultListOrderedDict):
        if isinstance(dom_or_dom2vars, (EVar, EVarArray)):  # meaning an alias
            elt.set(str(TypeXML.AS), dom_or_dom2vars.id)
        else:
            _text(elt, dom_or_dom2vars)
    else:
        for dom, vars in dom_or_dom2vars.items():
            s = compact(vars)
            elt.append(_element(TypeXML.DOMAIN, attributes=(TypeXML.FOR, " ".join(str(x) for x in s) if isinstance(s, list) else s), text=dom))
    if va.get_type() == TypeVar.SYMBOLIC:
        elt.set(str(TypeXML.TYPE), str(TypeVar.SYMBOLIC))
    return elt


def _simple_var(va, dom, dom2var):
    if dom in dom2var:
        return _complex_var(va, dom if len(dom) < SIZE_LIMIT_FOR_USING_AS else dom2var[dom])
    else:
        dom2var[dom] = va
        return _complex_var(va, dom)


def _variables():
    elt = _element(TypeXML.VARIABLES)
    dom2var = dict()
    for va in VarEntities.items:
        if isinstance(va, EVar):
            elt.append(_simple_var(va, str(va.variable.dom), dom2var))
        else:
            dom2vars = DefaultListOrderedDict(())
            for x in va.flatVars:
                if x is not None:
                    dom2vars[str(x.dom)].append(x)
            dom2vars = DefaultListOrderedDict(sorted(dom2vars.items(), key=lambda item: [y.indexes for y in item[1]]))
            if len(dom2vars) == 1:  # and not va.is_containing_hole():  # TODO do we keep the second part of the condition?
                elt.append(_simple_var(va, str(va.flatVars[0].dom), dom2var))
            else:
                elt.append(_complex_var(va, dom2vars))
    return elt


def _argument(elt, arg, key, value):
    assert value is not None
    if arg.lifted is True:  # TODO do we have an example? (3-tuples for attributes in this case?)
        for i, l in enumerate(value):
            subelt = _element(key, text=l)
            for att in (att for att in arg.attributes if att[1] == key and att[0] == i):
                subelt.set(str(att[2]), str(att[3]))
            elt.append(subelt)
    else:
        elt.append(_element(key, attributes=arg.attributes, text=None if isinstance(value, list) and len(value) == 1 and value[0] is None else value))


def _constraint(entity, *, possible_simplified_form=False):
    assert isinstance(entity, (ECtr, EObjective, EAnnotation))
    c = entity.constraint
    if c is None:
        return None
    if isinstance(c, ConstraintIntension):
        return _element(TypeCtr.INTENSION, entity, text=c.arguments[TypeCtrArg.FUNCTION].content)
    elt = _element(c.name, entity, attributes=c.attributes)
    arguments = [arg for arg in c.arguments.values() if arg.content is not None]  # we keep only valid (non null) arguments
    if len(arguments) == 1 and not arguments[0].lifted and (possible_simplified_form or arguments[0].name == TypeCtrArg.LIST):
        _text(elt, arguments[0].content)
    else:
        for arg in arguments:
            _argument(elt, arg, arg.name, arg.content)
    return elt


def _constraint_template(group):
    if isinstance(group.entities[0].constraint, ConstraintIntension):
        return _element(TypeCtr.INTENSION, text=group.abstraction)
    else:
        first = group.entities[0]
        arguments = [(k, v) for k, v in group.abstraction.items() if v is not None]  # we keep only valid (non null) arguments
        elt = _element(first.constraint.name, first, attributes=first.constraint.attributes)
        if len(arguments) == 1 and TypeCtrArg.LIST in group.abstraction:
            _text(elt, group.abstraction[TypeCtrArg.LIST])
        else:
            for k, v in arguments:
                _argument(elt, first.constraint.arguments[k], k, v)
        return elt


def _slide(entity, scope, offset, circular, group):
    elt = _element(TypeCtr.SLIDE, entity, attributes=(TypeXML.CIRCULAR, "true") if circular else [])
    elt.append(_element(TypeCtrArg.LIST, text=scope, attributes=(TypeXML.OFFSET, offset) if offset > 1 else []))
    elt.append(_constraint_template(group))
    return elt


def _group(entity):
    if options.recognizeslides:
        all_args = entity.original_all_args if hasattr(entity, "original_all_args") else entity.all_args
        if len(all_args) > 3 and len(all_args[0]) > 1:
            res = _identify_slide(entity)
            if res:
                scope, offset, circular = res
                return _slide(entity, compact(scope, preserve_order=True), offset, circular, entity)
    elt = _element(TypeXML.GROUP, entity)
    elt.append(_constraint_template(entity))
    for arg in entity.all_args:
        elt.append(_element(TypeXML.ARGS, text=arg))
    return elt


def _constraints_recursive(elt, entity):
    if entity is None:
        return None
    son = None
    if isinstance(entity, ECtr):
        son = _constraint(entity)
    elif isinstance(entity, EToGather):
        _constraints_iterative(elt, entity.entities)
    elif isinstance(entity, ESlide):
        if len(entity.scope) == 0:
            _constraints_iterative(elt, entity.entities)
        else:
            son = _slide(entity, entity.scope, entity.offset, entity.circular, entity.entities[0].entities[0])
    elif isinstance(entity, EGroup):
        son = _group(entity)
    elif isinstance(entity, EBlock):
        if len(entity.entities) != 0:
            son = _constraints_iterative(_element(TypeXML.BLOCK, entity), entity.entities)
    elif isinstance(entity, EIfThenElse):
        son = _constraints_iterative(_element(TypeXML.ifThenElse, entity), entity.entities)
    elif isinstance(entity, EToSatisfy):
        _constraints_iterative(elt, entity.entities)
    else:
        raise TypeError("Problem with " + str(type(entity)))
    if son is not None:
        elt.append(son)


def _constraints_iterative(elt, entities):
    for ce in entities:
        _constraints_recursive(elt, ce)
    return elt


def _constraints():
    return _constraints_iterative(_element(TypeXML.CONSTRAINTS), CtrEntities.items)


def _objectives():
    elt = _element(TypeXML.OBJECTIVES)
    for ce in ObjEntities.items:
        elt.append(_constraint(ce, possible_simplified_form=True))
    return elt


def _annotations():
    elt = _element(TypeXML.ANNOTATIONS)
    for ce in AnnEntities.items:
        elt.append(_constraint(ce, possible_simplified_form=True))
    return elt


def build_document():
    root = _element(TypeXML.INSTANCE, attributes=(TypeXML.FORMAT, "XCSP3"))

    variables = _variables()
    if len(variables) > 0:
        root.append(variables)
    else:
        print("Warning: no variables in this model (and so, no generated file)!")
        return None

    constraints = _constraints()
    if len(constraints) > 0:
        root.append(constraints)
    else:
        print("Warning: no constraints for this model!")

    objectives = _objectives()
    if len(objectives) > 0:
        root.append(objectives)
        root.set(str(TypeXML.TYPE), str(TypeFramework.COP))
    else:
        root.set(str(TypeXML.TYPE), str(TypeFramework.CSP))

    annotations = _annotations()
    if len(annotations) > 0:
        root.append(annotations)

    return root



"""
See https://en.wikipedia.org/wiki/Shikaku
See "Shikaku as a Constraint Problem" by Helmut Simonis

Example of Execution:
  python3 Shikaku.py -data=Shikaku_grid1.json
"""

from pycsp3 import *

nRows, nCols, rooms = data
nRooms = len(rooms)


def no_overlapping(i, j):
    leftmost = i if rooms[i].col <= rooms[j].col else j
    rightmost = j if leftmost == i else i
    p = r[leftmost] <= l[rightmost]
    if rooms[leftmost].row == rooms[rightmost].row:
        return p
    if rooms[leftmost].row > rooms[rightmost].row:
        return p | (t[leftmost] >= b[rightmost])
    return p | (b[leftmost] <= t[rightmost])


# l[i] is the position of the left border of the ith room
l = VarArray(size=nRooms, dom=range(nCols + 1))

# r[i] is the position of the right border of the ith room
r = VarArray(size=nRooms, dom=range(nCols + 1))

# t[i] is the position of the top border of the ith room
t = VarArray(size=nRooms, dom=range(nRows + 1))

# b[i] is the position of the bottom border of the ith room
b = VarArray(size=nRooms, dom=range(nRows + 1))

satisfy(
    # each room must be surrounded by its borders
    [(l[i] <= col, r[i] > col, t[i] <= row, b[i] > row) for i, (row, col, _) in enumerate(rooms)],

    # respecting the surface of each room
    [(r[i] - l[i]) * (b[i] - t[i]) == val for i, (_, _, val) in enumerate(rooms)],

    # rooms must not overlap
    [no_overlapping(i, j) for i, j in combinations(range(nRooms), 2)]
)



# [l[i] <= rooms[i].col for i in range(nRooms)],
#
# [r[i] > rooms[i].col for i in range(nRooms)],
#
# [t[i] <= rooms[i].row for i in range(nRooms)],
#
# [b[i] > rooms[i].row for i in range(nRooms)],



import os

from pycsp3.solvers.solver import SolverProcess, SolverPy4J, directory_of_solver, class_path_chocosolver


class ChocoProcess(SolverProcess):
    def __init__(self):
        c = class_path_chocosolver()
        super().__init__(name="Choco-solver", command="java -cp " + c + " org.chocosolver.parser.xcsp.ChocoXCSP")


class ChocoPy4J(SolverPy4J):
    def __init__(self):
        d = directory_of_solver("chocosolver")
        c = class_path_chocosolver()
        super().__init__(name="Choco-solver", command="java -cp " + c + os.pathsep + d + "py4j0.10.8.1.jar" + os.pathsep + d + " ChocoSolverPy4J")

# command="java -cp /usr/local/share/py4j/py4j0.10.8.1.jar:.:./pyAbsCon/ StackEntryPoint"



import sys
import types
from collections.abc import Iterable
from decimal import Decimal
from multiprocessing import cpu_count, Pool
from time import time

from pycsp3.classes.main.domains import Domain
from pycsp3.dashboard import options


class Stopwatch:
    def __init__(self):
        self.initial_time = time()

    def elapsed_time(self, *, reset=False):
        elapsed_time = time() - self.initial_time
        if reset:
            self.initial_time = time()
        return "{:10.2f}".format(elapsed_time)


class _Star(float):
    def __init__(self, val):
        super().__init__()

    def __repr__(self):
        return "*"

    def __str__(self):
        return "*"


ANY = _Star("Inf")  #: used to represent * in short tables


def flatten(*args, keep_none=False):
    # if not hasattr(flatten, "cache"):  # cannot work (changing to TupleInt and TupleVar instead of ListInt and ListVar while guaranteeing the lifetime? how?)
    #     flatten.cache = {}
    # elif len(args) == 1 and id(args[0]) in flatten.cache:
    #     return flatten.cache[id(args[0])]
    t = []
    for arg in args:
        if arg is None:
            if keep_none:
                t.append(arg)
        elif isinstance(arg, (str, range, Domain)):  # Iterable but must be appended, not extended
            t.append(arg)
        elif isinstance(arg, Iterable):
            t.extend(flatten(*arg, keep_none=keep_none))
        elif isinstance(arg, types.GeneratorType):
            t.extend(flatten(*list(arg), keep_none=keep_none))
        else:
            t.append(arg)
    # if len(args) == 1:
    #     flatten.cache[id(args[0])] = t
    return t


def is_containing(l, types, *, check_first_only=False):
    if isinstance(l, (list, tuple, set, frozenset)):
        if len(l) == 0:
            return None
        found = False
        for v in l:
            if not is_containing(v, types, check_first_only=check_first_only):
                return False
            if check_first_only:
                return True
            found = True
        return True if found else None
    else:
        return isinstance(l, types)


def unique_type_in(l, tpe=None):
    if isinstance(l, (list, tuple, set, frozenset)):
        if len(l) == 0:
            return None
        for v in l:
            t = unique_type_in(v, tpe)
            if t is False:
                return False
            if tpe is None:
                tpe = t
        return tpe
    else:
        return None if l is None else type(l) if tpe is None else tpe if isinstance(l, tpe) else False


def is_1d_list(l, types=None):
    return isinstance(l, list) and all(isinstance(v, types) if types else not isinstance(v, list) for v in l)


def is_1d_tuple(l, types):
    return isinstance(l, tuple) and all(isinstance(v, types) for v in l)


def is_2d_list(m, types=None):
    return isinstance(m, list) and all(is_1d_list(l, types) for l in m)


def is_matrix(m, types=None):
    return is_2d_list(m, types) and all(len(l) == len(m[0]) for l in m)


def is_square_matrix(m, types=None):
    return is_matrix(m, types) and len(m) == len(m[0])


def transpose(m):
    assert is_matrix(m)
    return [[m[j][i] for j in range(len(m))] for i in range(len(m[0]))]


def alphabet_positions(s):
    assert isinstance(s, str)
    return tuple(ord(c) - ord('a') for c in s.lower())


def all_primes(limit):
    """ Returns a list of primes < limit """
    sieve = [True] * limit
    for i in range(3, int(limit ** 0.5) + 1, 2):
        if sieve[i]:
            sieve[i * i::2 * i] = [False] * ((limit - i * i - 1) // (2 * i) + 1)
    return [2] + [i for i in range(3, limit, 2) if sieve[i]]


def value_in_base(decimal_value, length, base):
    assert type(decimal_value) == type(length) == type(base) is int
    value = [0] * length
    for i in range(len(value) - 1, -1, -1):
        value[i] = decimal_value % base
        decimal_value = decimal_value // base
    assert decimal_value == 0, "The given array is too small to contain all the digits of the conversion"
    return value


def integer_scaling(values):  # convert all (possibly decimal) specified values into integers by means of scaling
    values = list(values) if isinstance(values, types.GeneratorType) else values
    scale = 0
    for v in values:
        pos = v.find('.')
        if pos >= 0:
            i = len(v) - 1
            while v[i] == '0':
                i -= 1
            if i - pos > scale:
                scale = i - pos
    return [int(w * (10 ** scale)) for w in [Decimal(v) for v in values]]


def matrix_to_string(m):
    return "".join(["(" + ",".join([str(v) for v in t]) + ")" for t in m])
    # return "\n" + "\n".join(["\t(" + ",".join([str(v) for v in t]) + ")" for t in m]) + "\n"


def transitions_to_string(ts):
    return "".join(["(" + q1 + "," + str(v) + "," + q2 + ")" for (q1, v, q2) in ts])


def table_to_string(table, *, parallel=False):
    if not parallel or len(table) < 100000:
        s = []
        previous = ""
        for t in table:  # table is assumed to be sorted (adding an assert?) ; only distinct tuples are kept
            if t != previous:
                s.append("(" + ",".join(str(v) for v in t) + ")")
                previous = t
        return "".join(s)
    else:
        print("Parallel creation of a table of size: " + str(len(table)))
        n_threads = cpu_count()
        size = len(table) // n_threads
        pool = Pool(n_threads)
        left, right = 0, size
        t = []
        for piece in range(n_threads):
            t.append(pool.apply_async(table_to_string, args=(table[left:right],)))  # call not in parallel
            left += size
            right = len(table) if piece in {n_threads - 2, n_threads - 1} else right + size
        assert right == len(table)
        pieces = [r.get() for r in t]
        pool.close()
        pool.join()
        # checking and removing similar tuples before returning the string ?
        previous = None
        for piece in pieces:
            if previous and previous[-1] == piece[0]:
                previous.pop()
            previous = piece
        return "".join("".join(piece) for piece in pieces)


def integers_to_string(numbers):
    if len(numbers) == 0:
        return ''
    numbers = sorted(numbers)
    t = list()
    prev = numbers[0]
    for curr in numbers:
        if curr != prev + 1:
            t.append([curr])  # the start of a possible interval
        elif len(t[-1]) > 1:
            t[-1][-1] = curr  # to modify the end of the interval
        else:
            t[-1].append(curr)  # to set the end of the interval
        prev = curr
    return ' '.join(str(i[0]) if len(i) == 1 else str(i[0]) + ('..' if i[0] != i[1] - 1 else ' ') + str(i[1]) for i in t)


def display_constraints(ctr_entities, separator=""):
    for ce in ctr_entities:
        if ce is not None:
            if hasattr(ce, "entities"):
                print(separator + str(ce))
                display_constraints(ce.entities, separator + "\t")
            else:
                print(separator + str(ce.constraint))


PURPLE, BLUE, GREEN, ORANGE, RED, WHITE, WHITE_BOLD, UNDERLINE = '\033[95m', '\033[94m', '\033[92m', '\033[93m', '\033[91m', '\033[0m', '\033[1m', '\033[4m'


def warning(message):
    print("  " + ORANGE + "Warning: " + WHITE + message)


def error(s):
    print("\n\t" + RED + "ERROR: " + WHITE, s, "\n")
    print("\t\t(add option -ev to your command if you want to see the trace of the error)\n")
    if options.ev:
        raise TypeError(s)
    else:
        sys.exit(0)


def error_if(test, s):
    if test:
        error(s)



"""
Problem 076 on CSPLib, and NumberJack example


Examples of Execution:
  python3 CostasArray.py
  python3 CostasArray.py -data=10
"""

from pycsp3 import *

n = data or 8

# x[i] is the row where is put the ith mark (on the ith column)
x = VarArray(size=n, dom=range(n))

satisfy(
    # all marks are on different rows (and columns)
    AllDifferent(x),

    # all displacement vectors between the marks must be different
    [AllDifferent(x[i] - x[i + d] for i in range(n - d)) for d in range(1, n - 1)]
)

# TODO how to break all symmetries?  x[0] <= math.ceil(n / 2), x[0] < x[-1], ... ?



import datetime
import json
import lzma
import os
import os.path
import sys  # for DataVisitor import ast, inspect
from collections import OrderedDict
from importlib import util

from lxml import etree

from pycsp3.dashboard import options
from pycsp3.problems.data import parsing
from pycsp3.tools.aggregator import build_similar_constraints
from pycsp3.tools.compactor import build_compact_forms
from pycsp3.tools.curser import OpOverrider, convert_to_namedtuples, is_namedtuple
from pycsp3.tools.slider import handle_slides
from pycsp3.tools.utilities import Stopwatch
from pycsp3.tools.xcsp import build_document

None_Values = ['None', '', 'null']  # adding 'none'?


class Compilation:
    string_model = None
    string_data = None
    model = None
    data = None
    solve = None
    stopwatch = None
    stopwatch2 = None
    done = False

    @staticmethod
    def load(console=False):
        _load(console=console)

    @staticmethod
    def compile():
        return _compile()


def _load_options():
    options.set_values("data", "dataparser", "dataexport", "variant", "checker", "solver")
    options.set_flags("dataexport", "compress", "ev", "display", "time", "noComments", "recognizeSlides", "solve")
    if options.checker is None:
        options.checker = "fast"
    assert options.checker in {"complete", "fast", "none"}
    options.parse(sys.argv[1:])


def _load_model():
    try:
        name = sys.argv[0]
        assert name.strip().endswith(".py"), "The first argument has to be a python file." + str(name)
        model_string = name[name.rfind(os.sep) + 1:name.rfind(".")]
        specification = util.spec_from_file_location("", name)
        model = util.module_from_spec(specification)
        return model, model_string
    except:
        usage("It was not possible to read the file: " + sys.argv[0])
        raise


def _load_data():
    def _load_data_sequence(raw_data):
        od = [None if v in None_Values else int(v) if v and v.isdigit() else v for v in raw_data]
        return OrderedDict([("f" + str(i), od[i]) for i, v in enumerate(raw_data)]), od
        # return DataVisitor(raw_data).visit(ast.parse(inspect.getsource(Compilation.model)))

    data = options.data
    compilation_data = OrderedDict()  # the object used for recording the data, available in the model
    if data is None:
        return compilation_data, ""
    if data.endswith(".json"):
        assert os.path.exists(data), "The file " + data + " does not exist (in the specified directory)."
        with open(data) as f:
            compilation_data = json.loads(f.read(), object_pairs_hook=OrderedDict)
            string_data = "-" + data.split(os.sep)[-1:][0].split(".")[:1][0]
    else:
        #  if '{' in data and '}' in data:
        #    compilation_data = json.loads(data, object_hook=lambda d: namedtuple('X', d.keys())(*d.values()), object_pairs_hook=OrderedDict)
        #    for k, v in compilation_data.items(): setattr(compilation_data, k, v)  ordered_data = list(compilation_data.values())
        if data[0] == '[' and data[-1] == ']' or data[0] == '(' and data[-1] == ')':  # NB: these characters may be needed to be escaped as in \[2,3\]
            args = data[1:-1].split(",")
            if '=' in data:
                assert data.count('=') == data.count(',') + 1, "badly formed string of data " + data
                ordered_data = []
                for arg in args:
                    t = arg.split('=')
                    value = None if t[1] in None_Values else int(t[1]) if t[1].isdigit() else t[1]
                    compilation_data[t[0]] = value
                    ordered_data.append(value)
            else:
                compilation_data, ordered_data = _load_data_sequence(args)
        else:
            compilation_data, ordered_data = _load_data_sequence([data])
        string_data = "-" + "-".join(str(v) for v in ordered_data)
    return compilation_data, string_data


def _load_dataparser(parser_file, data_file):
    try:
        compilation_data = parsing.register_fields(data_file)  # the object used for recording data is returned, available in the model
        specification = util.spec_from_file_location("", parser_file)
        specification.loader.exec_module(util.module_from_spec(specification))
        string_data = "-" + options.data.split(os.sep)[-1:][0].split(".")[:1][0] if options.data else None
        if string_data is None:
            string_data = Compilation.string_data if Compilation.string_data else ""  # in case data are recorded through the dataparser (after asking the user)
        return compilation_data, string_data
    except:
        usage("It was not possible to correctly read the file: " + parser_file)
        raise


def _load(*, console=False):
    Compilation.stopwatch = Stopwatch()
    _load_options()
    if console is False:
        Compilation.model, Compilation.string_model = _load_model()
        if options.dataparser:
            Compilation.data, Compilation.string_data = _load_dataparser(options.dataparser, options.data)
        else:
            Compilation.data, Compilation.string_data = _load_data()
        Compilation.data = convert_to_namedtuples(Compilation.data)
        if len(Compilation.data) == 0:
            Compilation.data = None
        elif len(Compilation.data) == 1:
            Compilation.data = Compilation.data[0]  # the value instead of a tuple of size 1
    else:
        Compilation.string_model = "Console"
        Compilation.string_data = ""
    OpOverrider.enable()
    options.time and print("\tWCK for loading model and data:", Compilation.stopwatch.elapsed_time(), "seconds")


def default_data(filename):
    fn = os.path.dirname(os.path.realpath(__file__)) + os.sep + "problems" + os.sep + "data" + os.sep + "json" + os.sep + filename
    assert fn.endswith(".json")
    assert os.path.exists(fn), "The file " + fn + " does not exist (in the specified directory)."
    with open(fn) as f:
        Compilation.data = convert_to_namedtuples(json.loads(f.read(), object_pairs_hook=OrderedDict))
        Compilation.string_data = "-" + filename.split(os.sep)[-1:][0].split(".")[:1][0]
    if len(Compilation.data) == 1:
        Compilation.data = Compilation.data[0]  # the value instead of a tuple of size 1
    return Compilation.data


def _compile():
    # used to save data in jSON
    def prepare_for_json(obj):
        if is_namedtuple(obj):
            r = obj._asdict()
            for k in r:
                r[k] = prepare_for_json(r[k])
            return r
        if isinstance(obj, list):
            for i in range(len(obj)):
                obj[i] = prepare_for_json(obj[i])
            return obj
        return str(obj) if isinstance(obj, datetime.time) else obj

    OpOverrider.disable()
    if options.display:
        # print("\n", sys.argv, "\n")
        if sys.argv[1].endswith(".json"):
            with open(sys.argv[1], 'r') as f:
                print(f.read())

    stopwatch = Stopwatch()
    build_similar_constraints()
    options.time and print("\tWCK for generating groups:", stopwatch.elapsed_time(reset=True), "seconds")
    handle_slides()
    options.time and print("\tWCK for handling slides:", stopwatch.elapsed_time(reset=True), "seconds")
    build_compact_forms()
    options.time and print("\tWCK for compacting forms:", stopwatch.elapsed_time(reset=True), "seconds")

    filename_prefix = Compilation.string_model + ("-" + options.variant if options.variant else "") + Compilation.string_data

    filename = filename_prefix + ".xml"
    root = build_document()
    if root is not None:
        pretty_text = etree.tostring(root, pretty_print=True, xml_declaration=False).decode("UTF-8")
        with open(filename, "w") as f:
            f.write(pretty_text)
            print("  Generation of the file " + filename + " completed.")
        if options.compress:
            with lzma.open(filename + ".lzma", "w") as f:
                f.write(bytes(pretty_text, 'utf-8'))
                print("\tGeneration of the file " + filename + ".lzma completed.")
        if options.display:
            print("\n", pretty_text)
        options.time and print("\tWCK for generating files:", stopwatch.elapsed_time(reset=True), "seconds")

    if options.dataexport:
        if isinstance(options.dataexport, bool):
            json_prefix = options.data.split("/")[-1:][0].split(".")[:1][0] if options.dataparser else filename_prefix
            # TODO if data are given with name as e.g., in [k=3,l=9,b=0,r=0,v=9] for Bibd, maybe we should sort them
        else:
            json_prefix = str(options.dataexport)
        with open(json_prefix + '.json', 'w') as f:
            json.dump(prepare_for_json(Compilation.data), f)
        print("  Generation for data saving of the file " + json_prefix + '.json' + " completed.")

    print("\tTotal wall clock time:", Compilation.stopwatch.elapsed_time(), "seconds")

    Compilation.done = True

    if options.solve or options.solver:
        if options.solver == "choco":
            from pycsp3.solvers.chocosolver import ChocoProcess
            solution = ChocoProcess().solve(filename)
        else:  # Fallback case => options.solver == "abscon":
            from pycsp3.solvers.abscon import AbsConProcess
            solution = AbsConProcess().solve(filename)

        print()
        print(solution)

    return filename


def usage(message):
    print(message)
    print("\nThe PyCSP3 Compiler allows us to generate XCSP3 files.")
    print("\n\nUsage: python3.5 <model> <data>")
    print("  - <model> is the name of a Python file containing a PyCSP3 model (i.e., a Python file with code posting variables/constraints/objectives)")
    print("  - <data> is either a fixed list of elementary data or the name of a JSON file")



import re
from collections import OrderedDict

from pycsp3.dashboard import options

data = None
_data_file = None
_dataParser = None


def register_fields(data_file):
    global data, _data_file, _dataParser
    data = OrderedDict()
    _data_file = data_file
    _dataParser = DataParser(data_file)
    return data


class DataParser:
    def __init__(self, data_file):
        if data_file:
            with open(data_file) as f:
                self.lines = [line[:-1].strip() if line[-1] == '\n' else line.strip() for line in f.readlines() if len(line.strip()) > 0]
        self.curr_line_index = 0
        self.curr_line_tokens = None

    def curr_line(self):
        if self.curr_line_index >= len(self.lines):
            print("Warning: no more line")
            return None
        return self.lines[self.curr_line_index]

    def next_line(self):
        self.curr_line_index += 1
        self.curr_line_tokens = None
        return self.curr_line()

    def next(self, to_int=True):
        if self.curr_line_tokens is None:
            if self.curr_line() is not None:
                self.curr_line_tokens = self.curr_line().split()
                self.curr_line_tokens_index = 0
        res = int(self.curr_line_tokens[self.curr_line_tokens_index]) if to_int else self.curr_line_tokens[self.curr_line_tokens_index]
        self.curr_line_tokens_index += 1
        if self.curr_line_tokens_index >= len(self.curr_line_tokens):
            next_line()
            while self.curr_line() is not None and len(self.curr_line().strip()) == 0:
                self.next_line()
        return res


def line():
    return _dataParser.curr_line()


def next_line(repeat=0):
    for _ in range(repeat + 1):
        l = _dataParser.next_line()
    return l


def next_int():
    return _dataParser.next(to_int=True)


def next_str():
    return _dataParser.next(to_int=False)


def remaining_lines(skip_curr=False):
    if skip_curr:
        next_line()
    return _dataParser.lines[_dataParser.curr_line_index:]


def number_in(line):
    assert line is not None
    return int(re.search(r'[-]?\d+', line).group(0))


def numbers_in(line):
    assert line is not None
    return [int(v) for v in re.findall(r'[-]?\d+', line)]  # [int(v) for v in line().split() if v.isdigit()]


def ask_number(message):
    parameter = options.consume_parameter()
    return int(parameter) if parameter else int(input(message + " "))


def ask_string(message):
    parameter = options.consume_parameter()
    return parameter if parameter else input(message + " ")



from pycsp3 import *

k, d = data.k, data.d

# x[i][j] is the value of the matrix at row i and column j
x = VarArray(size=[k, k], dom=set(range(-d, -1)) | {0} | set(range(2, d+1)))

det = Var(dom={-1, 1})

if k == 3:
    nodes = [
        x[0][0] * x[1][1] * x[2][2],
        x[0][0] * x[1][2] * x[2][1],
        x[0][1] * x[1][2] * x[2][0],
        x[0][1] * x[1][0] * x[2][2],
        x[0][2] * x[1][0] * x[2][1],
        x[0][2] * x[1][1] * x[2][0]
    ]
    satisfy(
        Sum(nodes * [1, -1, 1, -1, 1, -1]) == det
    )
    # TO BE continued


satisfy(
    # Increasingly ordering both rows and columns
    # tag(symmetryBreaking)
    LexIncreasing(x, matrix=True)
)




from pycsp3.problems.tests.tester import Tester, run

NAME = "g0_school"

diff = (Tester(NAME)
        )

same = (Tester(NAME)
        .add("Pic")
        .add("Pic2")
        .add("Riddle")
        .add("Riddle2")
        .add("Riddle3a")
        .add("Riddle3b")
        .add("Riddle4a")
        .add("Riddle4b")
        .add("Riddle5")
        .add("SimpleColoring")
        )

xcsp = (Tester(NAME)
        .add(same.instances)
        )

run(xcsp, diff, same)



from pyCSP3 import *

domains, routes = data.domains, data.routes
n = len(routes)
hards, softs = data.hards, data.softs
nHards, nSofts = len(hards) if hards else 0, len(softs)

cache = {}  # for tables


def frequency_domain(i):
    return set(domains[routes[i].domain])


def polarization_domain(i):
    return {0, 1} if routes[i].polarization == 0 else {1} if routes[i].polarization == 1 else {0}


def create_tuple():
    pass


def create_tuple_short_version():
    pass


def table_relaxable_link(c, short_version=False):
    OpOverrider.disable()
    dom1, dom2 = routes[c.route1].domain, routes[c.route2].domain
    pol1, pol2 = routes[c.route1].polarization, routes[c.route2].polarization
    key = str(dom1) + "*" + str(dom2) + "*" + str(pol1) + "*" + str(pol2) + "*" + ' '.join([str(e) for e in c.eqRelaxations]) + "*" + ' '.join(
        [str(e) for e in c.neRelaxations])
    if key in cache:
        return cache[key]
    set_short_version = set()
    table = []
    for f1 in domains[dom1]:
        for f2 in domains[dom2]:
            distance = abs(f1 - f2)
            if short_version and distance in set_short_version:
                continue
            for pol in range(4):
                p1 = 0 if pol < 2 else 1
                p2 = 1 if pol in {1, 3} else 0
                if (pol1, p1) in {(1, 0), (-1, 1)} or (pol2, p2) in {(1, 0), (-1, 1)}:
                    continue
                t = c.eqRelaxations if p1 == p2 else c.neRelaxations
                for k in range(12):
                    if k == 11 or distance >= t[k]:  # for k=11, we suppose t[k] = 0
                        size = len([l for l in range(0, k - 1) if distance < t[l]])
                        if short_version:
                            table.append((distance, p1, p2, k, 0 if k == 0 or distance >= t[k - 1] else 1, 0 if k <= 1 else size))
                        else:
                            table.append((f1, f2, p1, p2, k, 0 if k == 0 or distance >= t[k - 1] else 1, 0 if k <= 1 else size))
            set_short_version.add(distance)
    cache[key] = table
    OpOverrider.enable()
    return table


def distances(i, j):
    dom1, dom2 = domains[routes[i].domain], domains[routes[j].domain]
    return {abs(f1 - f2) for f1 in dom1 for f2 in dom2}


def soft_links():
    t = [[False] * n for _ in range(n)]
    for c in softs:
        i, j = c.route1, c.route2
        t[i][j] = True
        t[j][i] = True
    return t


def imperative(c):
    i, j = c.route1, c.route2
    if c.frequency:
        if c.gap == 0:
            return f[i] == f[j] if c.equality else f[i] != f[j]
        return dist(f[i], f[j]) == c.gap if c.equality else dist(f[i], f[j]) != c.gap
    return p[i] == p[j] if c.equality else p[i] != p[j]


softLinks = soft_links()  # used by model m2 (not currently implemented)

#  f[i] is the frequency of the ith radio-link
f = VarArray(size=[n], dom=frequency_domain)

#  p[i] is the polarization of the ith radio-link
p = VarArray(size=[n], dom=polarization_domain)

# k is the relaxation level to be optimized
k = Var(dom=range(12))

#  v1[q] is 1 iff the qth pair of radio-electric compatibility constraints is violated when relaxing another level
v1 = VarArray(size=[nSofts], dom={0, 1})

#  v2[q] is the number of times the qth pair of radio-electric compatibility constraints is violated when relaxing more than one level
v2 = VarArray(size=[nSofts], dom=range(11))

satisfy(
    # imperative constraints
    [imperative(h) for h in hards]
)

if not variant():
    satisfy(
        # relaxable radio-electric compatibility constraints
        (f[c.route1], f[c.route2], p[c.route1], p[c.route2], k, v1[i], v2[i]) in table_relaxable_link(c) for i, c in enumerate(softs)
    )

elif variant("short"):
    # d[i][j] is the distance between the ith and the jth frequencies (for i < j when a soft link exists)
    d = VarArray(size=[n, n], dom=lambda i, j: distances(i, j) if i < j and softLinks[i][j] else None)

    satisfy(

        # computing intermediary distances
        [d[i][j] == dist(f[i], f[j]) for i in range(n) for j in range(i + 1, n) if softLinks[i][j]],

        # relaxable radio-electric compatibility constraints
        [((d[c.route1][c.route2] if c.route1 < c.route2 else d[c.route2][c.route1]), p[c.route1], p[c.route2], k, v1[i], v2[i]) in table_relaxable_link(c, True)
         for i, c in enumerate(softs)]

    )

minimize(
    k * (10 * nSofts ** 2) + Sum(v1 * ([10 * nSofts] * nSofts)) + Sum(v2)
)



"""
From http://jimorlin.wordpress.com/2009/02/17/colored-letters-labeled-dice-a-logic-puzzle/
There are 13 words as follows: buoy, cave, celt, flub, fork, hemp, judy, junk, limn, quip, swag, visa.
There are 24 different letters that appear in the 13 words.
The question is: can one assign the 24 letters to 4 different cubes so that the four letters of each word appears on different cubes.
There is one letter from each word on each cube.
The puzzle was created by Humphrey Dudley.

Execution:
  python3 LabeledDice.py
"""

from pycsp3 import *

words = ["buoy", "cave", "celt", "flub", "fork", "hemp", "judy", "junk", "limn", "quip", "swag", "visa"]

# x[i] is the cube where the ith letter of the alphabet is put
x = VarArray(size=26, dom=lambda i: range(1, 5) if i in alphabet_positions("".join(words)) else None)

satisfy(
    # the four letters of each word appears on different cubes
    [AllDifferent(x[i] for i in alphabet_positions(w)) for w in words],

    # each cube is assigned 6 letters
    Cardinality(x, occurrences={i: 6 for i in range(1, 5)})
)



from pycsp3 import *

x = VarArray(size=[10, 10], dom=range(150))
y = VarArray(size=[10, 10], dom=range(150))
z = VarArray(size=10, dom=range(150))

satisfy(
    [Sum(x[p]) in range(100) for p in range(10)],
    [Sum(x[p]) in {1, 2, 3} for p in range(10)],
    [Sum(x[p]) not in range(100) for p in range(10)],
    [Sum(x[p]) not in {1, 2, 3} for p in range(10)],
    [Sum(x[p]) <= 5 for p in range(10)],
    [Sum(x[p]) < 5 for p in range(10)],
    [Sum(x[p]) >= 5 for p in range(10)],
    [Sum(x[p]) > 5 for p in range(10)],
    [Sum(x[p]) == 5 for p in range(10)],
    [Sum(x[p]) != 5 for p in range(10)],

    Sum(y) in range(50),
    Sum(y) not in range(50),
    Sum(y) in {1, 2, 3},
    Sum(y) not in {1, 2, 3},
    Sum(y) <= 5,
    Sum(y) < 5,
    Sum(y) >= 5,
    Sum(y) > 5,
    Sum(y) == 5,
    Sum(y) != 5
)



"""
See Problem 002 on CSPLib

Examples of Execution:
  python3 TemplateDesign.py -data=TemplateDesign_catfood_2.json
  python3 TemplateDesign.py -data=TemplateDesign_catfood_2.json -variant=aux
"""

from pycsp3 import *
from math import ceil, floor

nSlots, demands = data
nTemplates = nVariations = len(demands)


def variation_interval(v):
    return range(ceil(demands[v] * 0.95), floor(demands[v] * 1.1) + 1)


# d[i][j] is the number of occurrences of the jth variation on the ith template
d = VarArray(size=[nTemplates, nVariations], dom=range(nSlots + 1))

# p[i] is the number of printings of the ith template
p = VarArray(size=nTemplates, dom=range(max(demands) + 1))

satisfy(
    # all slots of all templates are used
    Sum(d[i]) == nSlots for i in range(nTemplates)
)

if not variant():
    satisfy(
        # respecting printing bounds for each variation
        p * d[:, j] in variation_interval(j) for j in range(nVariations)
    )

elif variant("aux"):
    # pv[i][j] is the number of printings of the jth variation by using the ith template
    pv = VarArray(size=[nTemplates, nVariations], dom=lambda i, j: range(variation_interval(j).stop))

    satisfy(
        # linking variables of arrays p and pv
        [p[i] * d[i][j] == pv[i][j] for i in range(nTemplates) for j in range(nVariations)],

        # respecting printing bounds for each variation v
        [Sum(pv[:, j]) in variation_interval(j) for j in range(nVariations)]
    )

satisfy(
    # tag(symmetry-breaking)
    [
        [iff(p[i] == 0, d[i][0] == nSlots) for i in range(nTemplates)],

        Decreasing(p),
    ]
)

minimize(
    # minimizing the number of used templates
    Sum(p[i] > 0 for i in range(nTemplates))
)



"""
See "Teaching Constraints through Logic Puzzles" by Peter Szeredi
See "Dominoes as a Constraint Problem" by Helmut Simonis

Example of Execution:
  python3 Dominoes.py -data=Dominoes_grid1.json
"""

from pycsp3 import *

grid = data
nRows, nCols, nValues = len(grid), len(grid[0]), len(grid)
positions = [[i * nCols + j for i in range(nRows) for j in range(nCols) if grid[i][j] == value] for value in range(nValues)]

# x[i][j] concerns the domino (having values) i-j; this is the position of the value i in the grid for this domino
x = VarArray(size=[nValues, nValues], dom=lambda i, j: range(nRows * nCols) if i <= j else None)

# y[i][j] concerns the domino (having values) i-j; this is the position of the value j in the grid for this domino
y = VarArray(size=[nValues, nValues], dom=lambda i, j: range(nRows * nCols) if i <= j else None)

satisfy(
    # each part of each domino in a different cell
    AllDifferent(x + y),

    # unary constraints
    [(x[i][j] in positions[i], y[i][j] in positions[j]) for i in range(nValues) for j in range(i, nValues)],

    # adjacency constraints
    [(abs(x[i][j] - y[i][j]) == nCols) | (abs(x[i][j] - y[i][j]) == 1) & (x[i][j] // nCols == y[i][j] // nCols)
     for i in range(nValues) for j in range(i, nValues)]
)



import re
from collections import defaultdict

from pycsp3.classes.auxiliary.ptypes import TypeCtrArg
from pycsp3.classes.entities import CtrEntities, ObjEntities, AnnEntities, ESlide, EGroup, VarEntities
from pycsp3.classes.main.variables import Variable
from pycsp3.tools.utilities import flatten, is_containing


class SequenceOfSuccessiveVariables:
    def __init__(self, x):
        self.firstVar, self.posMod, self.stopMod, self.arrayVar = x, -1, 0, VarEntities.varToEVarArray[x]
        if '[' in x.id:
            pos = x.id.index("[")
            self.prefix, self.suffix = x.id[:pos], x.id[pos:]
            self.starts = [int(v) for v in re.split("\]\[", self.suffix[1:-1])]  # the indexes of the first variable (in case of an array)

    def differ_just_at(self, t):
        pos = -1
        for i in range(len(self.starts)):
            if self.starts[i] != t[i]:
                if pos == -1:
                    pos = i
                else:
                    return -1  # at least two differences
        return pos

    def can_be_extended_with(self, id):
        x = Variable.name2obj[id]
        if not x.indexes or x.prefix != self.prefix:  # meaning x is not from an array
            return None
        t = x.indexes
        pos = self.differ_just_at(t)
        if pos == -1:
            return False
        if self.posMod == -1:
            if t[pos] != self.starts[pos] + 1:
                return False
            self.posMod, self.stopMod = pos, t[pos]
        else:
            if pos != self.posMod or t[pos] != self.stopMod + 1:
                return False
            self.stopMod = t[pos]
        return True

    def __repr__(self):
        s = self.prefix
        for i, value in enumerate(self.starts):
            if self.posMod != i:
                s += "[" + str(value) + "]"
            elif self.starts[self.posMod] == 0 and self.stopMod == self.arrayVar.size[self.posMod] - 1:
                s += "[]"
            else:
                s += "[" + str(value) + ".." + str(self.stopMod) + "]"
        return s


def build_partition(variables, preserve_order):
    def _simple_partition(vars):
        # if isinstance(vars, list) and len(vars) == 1 and isinstance(vars[0], list): vars = vars[0]  # TODO never reached. We can remove that ?
        harvest = []  # variables (and other objects) are collected to compute the complete expanded form (string) with their names (values)
        arrays = []  # arrays encountered and recorded all along the process
        others = []  # list collecting any stand-alone variables or any kind of objects that are not variables from arrays
        arrays_partition = defaultdict(list)  # a dict for partitioning variables according to arrays

        def update(z):
            harvest.append(z)
            if isinstance(z, Variable) and z.indexes:  # if z from an array
                va = VarEntities.varToEVarArray[z]
                arrays_partition[va].append(z)
                if va not in arrays:
                    arrays.append(va)
                elif arrays[-1][0] != va:
                    return False
            else:  # if z is not DELETED:  # it can be a Variable, but also an integer, a Node, a state (for an automaton).
                others.append(z)
            return True

        for x in vars:
            if isinstance(x, list):
                for y in x:
                    assert isinstance(y, Variable), "a variable is expected in the inner list. Problem with " + str(
                        y)  # relaxing this by a test 'not isinstance(y,list)' ?
                    if not update(y):
                        return None  # because variables from different arrays are mixed
            else:
                if not update(x):
                    return None  # because variables from different arrays are mixed
        # arrays = sorted(arrays, key=attrgetter('id'))  # TODO useless. No impact at all. We can remove that, right?
        return arrays, others, arrays_partition, " ".join(str(x) for x in harvest)

    def _complex_partition(vars):
        harvest = []  # variables (and other objects) are collected to compute the complete expanded form (string) with their names (values)
        partition = []  # the partition built from successive identified parts
        part = []  # the current part being built (before being added to the partition)
        no_arrays = False

        def update(z):
            nonlocal part, no_arrays
            harvest.append(z)
            if isinstance(z, Variable) and z.indexes:
                no_arrays = False
                if len(part) == 0 or VarEntities.varToEVarArray[z].id == VarEntities.varToEVarArray[part[-1]].id:
                    part.append(z)
                else:
                    partition.append(part)
                    part = [z]
            else:  # z is not DELETED:
                partition.append(part)
                partition.append(z)
                part = []

        for x in vars:
            if isinstance(x, list):
                for y in x:
                    update(y)
            else:
                update(x)
        if len(part) > 0:
            partition.append(part)
        if no_arrays:
            return None, None
        return partition, " ".join(str(x) for x in harvest)

    if not preserve_order:
        simple_partition = _simple_partition(variables)  # we make an attempt to build a simple partition
    else:
        simple_partition = None
    if simple_partition:
        arrays, others, arrays_partition, complete_expanded_form = simple_partition
        if len(arrays) == 0:
            return None, None
        # Here, as the order is not a problem, we can below build the partition by means of the returned structure
        return [arrays_partition[a] for a in arrays] + [others], complete_expanded_form
    else:
        return _complex_partition(variables)


def _complex_compact(variables):
    compact_form = ""
    sequence = None
    for x in variables:
        if sequence is None:
            sequence = SequenceOfSuccessiveVariables(x)
        elif not sequence.can_be_extended_with(x.id):
            compact_form += str(sequence) if compact_form == "" else " " + str(sequence)
            sequence = SequenceOfSuccessiveVariables(x)
    compact_form += str(sequence) if compact_form == "" else " " + str(sequence)
    return compact_form


''' We try to identify a compact form corresponding to a unique token (note that variables are from the same array, by construction) '''


def _simple_compact(variables):
    var_array = VarEntities.varToEVarArray[variables[0]]
    mins, maxs = [float('inf')] * len(var_array.size), [float('-inf')] * len(var_array.size)
    for x in variables:
        for i, v in enumerate(x.indexes):
            mins[i] = min(mins[i], v)
            maxs[i] = max(maxs[i], v)
    # Check the size
    size = 1
    for i in range(len(mins)):
        size *= maxs[i] - mins[i] + 1
    if size != len(variables):
        return None  # because it means that we didn't succeed in having a simple compact form (i.e., a unique token)

    compact_form = var_array.id
    for i in range(len(mins)):
        compact_form += "["
        if (mins[i], maxs[i]) != (0, var_array.size[i] - 1):
            compact_form += str(mins[i]) if mins[i] == maxs[i] else str(mins[i]) + ".." + str(maxs[i])
        compact_form += "]"
    return compact_form


def _expand(compact_form):
    assert " " not in compact_form, "The specified string must correspond to a single token; bad form : " + compact_form
    if compact_form[-1] == ")":
        return compact_form  # // this means that we have an expression (predicate) here
    if "[" not in compact_form:
        return compact_form
    pos = compact_form.index("[")
    prefix, suffix = compact_form[:pos], compact_form[pos:]
    tokens = [int(v) if v.isdigit() else v for v in re.split("\]\[", suffix[1:-1])]
    var_array = VarEntities.prefixToEVarArray[prefix]
    assert var_array, "Pb with " + compact_form
    assert len(var_array.size) == len(tokens)
    mins, maxs, sizes = [0] * len(tokens), [0] * len(tokens), [0] * len(tokens)
    for i, value in enumerate(tokens):
        if isinstance(value, int):
            mins[i] = maxs[i] = value
        elif value == "":
            mins[i] = 0
            maxs[i] = var_array.size[i] - 1
        else:
            spl = value.split("..")
            mins[i], maxs[i] = int(spl[0]), int(spl[1])
        sizes[i] = maxs[i] - mins[i] + 1

    var_names = flatten(Variable.build_names_array(prefix, sizes, mins))
    return " ".join(s for s in var_names)


def compact(variables, *, preserve_order=False, group_args=False):
    if not isinstance(variables, list):
        return variables
    if group_args is False and len(variables) < 3:
        return variables
    partition, complete_expanded_form = build_partition(variables, preserve_order)
    if partition is None:
        return variables
    t = []
    for part in partition:
        if isinstance(part, list):
            if not preserve_order:
                part = sorted(part, key=lambda x: [int(v) for v in re.split("\]\[", x.id[x.id.index("[") + 1:-1])])
            if len(part) > 2:
                compact = _simple_compact(part)
                if compact is None:
                    t.append(_complex_compact(part))
                else:
                    if preserve_order:
                        expand = " ".join([_expand(e) for e in compact.split()])
                        if expand not in complete_expanded_form:
                            t.append(_complex_compact(part))
                        else:
                            t.append(compact)
                    else:
                        t.append(compact)
            else:
                t.extend(part)
        else:
            t.append(part)
    return " ".join(str(s) for s in t)


def _compact_constraint_arguments(arguments):
    for arg in list(arguments.values()):
        if isinstance(arg.content, list) and len(arg.content) > 0 and arg.content_compressible:
            if not isinstance(arg.content[0], list):  # It is only one list
                arg.content = compact(arg.content, preserve_order=arg.content_ordered)
            elif arg.lifted is True:
                arg.content = [compact(l, preserve_order=arg.content_ordered) for l in arg.content]
        elif arg.name == TypeCtrArg.MATRIX:  # Special case for matrix
            sc = None if is_containing(arg.content_compressible, int) else _simple_compact(flatten(arg.content_compressible))
            arg.content = sc if sc is not None else arg.content


def _compact_constraint_group(group):
    preserve_order = False
    cnt = 0
    if isinstance(group.abstraction, dict):
        for key, value in group.abstraction.items():
            if "%" in str(value):
                cnt += 1
            argument = group.entities[0].constraint.arguments[key]
            if argument.content_compressible:
                if "%" not in str(value):
                    if key == TypeCtrArg.MATRIX:  # Special case for matrix
                        sc = None if is_containing(argument.content_compressible, int) else _simple_compact(flatten(argument.content_compressible))
                        group.abstraction[key] = sc if sc is not None else group.abstraction[key]
                    else:
                        group.abstraction[key] = compact(value, preserve_order=argument.content_ordered)
                elif argument.content_ordered is True:
                    preserve_order = True
    else:
        preserve_order = True
    if cnt > 1:
        preserve_order = True
    group.original_all_args = [aa for aa in group.all_args]  # useful when reasoning to build the meta-constraint 'slide'
    for i in range(len(group.all_args)):
        group.all_args[i] = compact(group.all_args[i], preserve_order=preserve_order, group_args=True)


def _compact_forms_recursive(entities):
    for e in entities:
        if e is not None:
            if hasattr(e, "entities"):
                if isinstance(e, EGroup):
                    _compact_constraint_group(e)
                elif isinstance(e, ESlide):
                    if len(e.scope) > 0:
                        e.scope = compact(e.scope)
                    _compact_forms_recursive(e.entities)
                else:
                    _compact_forms_recursive(e.entities)
            elif e.constraint is not None:
                _compact_constraint_arguments(e.constraint.arguments)


def build_compact_forms():
    _compact_forms_recursive(CtrEntities.items)
    _compact_forms_recursive(ObjEntities.items)
    _compact_forms_recursive(AnnEntities.items)



"""
See Problem 056 on CSPLib

Example of Execution:
  python3 Sonet.py -data=Sonet_sonet1.json
"""

from pycsp3 import *

m, n, r, connections = data

# x[i][j] is 1 if the ith ring contains the jth node
x = VarArray(size=[m, n], dom={0, 1})

table = {tuple(1 if j // 2 == i else ANY for j in range(2 * m)) for i in range(m)}

satisfy(
    [(x[i][j1] if k == 0 else x[i][j2] for i in range(m) for k in range(2)) in table for (j1, j2) in connections],

    [Count(x[i], value=1) <= r for i in range(m)],

    LexIncreasing(x)
)

minimize(
    Sum(x)
)



"""
See QAPLib and https://en.wikipedia.org/wiki/Quadratic_assignment_problem

Example of Execution:
  python3 QuadraticAssignment.py -data=QuadraticAssignment_qap.json
  python3 QuadraticAssignment.py -data=QuadraticAssignment_example.txt -dataparser=QuadraticAssignment_Parser.py
"""

from pycsp3 import *

weights, distances = data  # facility weights and location distances
all_distances = {d for row in distances for d in row}
n = len(weights)

table = {(i, j, distances[i][j]) for i in range(n) for j in range(n) if i != j}

# x[i] is the location assigned to the ith facility
x = VarArray(size=n, dom=range(n))

# d[i][j] is the distance between the locations assigned to the ith and jth facilities
d = VarArray(size=[n, n], dom=lambda i, j: all_distances if i < j and weights[i][j] != 0 else None)

satisfy(
    # all locations must be different
    AllDifferent(x),

    # computing the distances
    [(x[i], x[j], d[i][j]) in table for i, j in combinations(range(n), 2) if weights[i][j] != 0]
)

minimize(
    #  minimizing summed up distances multiplied by flows
    Sum(d[i][j] * weights[i][j] for i, j in combinations(range(n), 2) if weights[i][j] != 0)
)



import types
from collections import deque, namedtuple

from pycsp3.classes.entities import Node, TypeNode
from pycsp3.classes.main.constraints import (
    ScalarProduct, PartialConstraint, ConstraintSum, ConstraintElement, ConstraintElementMatrix, ConstraintInstantiation, ECtr, auxiliary)
from pycsp3.classes.main.variables import Variable, VariableInteger, NotVariable
from pycsp3.libs.forbiddenfruit import curse
from pycsp3.tools.inspector import checkType
from pycsp3.tools.utilities import flatten, is_containing, unique_type_in, is_1d_tuple, is_1d_list, is_2d_list, is_matrix, ANY, error_if

queue_in = deque()  # To store partial constraints when using the IN operator

unsafe_cache = False  # see for example Pic since the table is released as it occurs as a parameter


def cursing():
    def _dict_add(self, other):  # for being able to merge dictionaries
        if isinstance(other, dict):
            d = self.copy()
            d.update(other)
            return d
        raise NotImplementedError  # return save_dict_add(self, other)

    def _tuple_mul(self, other):  # for being able to use scalar products
        if is_containing(self, (Variable, Node), check_first_only=True):
            return ScalarProduct(self, other)
        if is_containing(self, int) and isinstance(other, (list, tuple)) and is_containing(other, (Variable, Node), check_first_only=True):
            return ScalarProduct(other, self)
        return tuple.__mul__(self, other)

    def _list_mul(self, other):  # for being able to use scalar products
        if is_containing(self, (Variable, Node), check_first_only=True):
            return ScalarProduct(self, other)
        return list.__mul__(self, other)

    def _tuple_contains(self, other):
        if not OpOverrider.activated:
            return self.__contains__(other)
        if is_containing(other, Variable) and len(self) > 0 and isinstance(self[0], (tuple, int)):
            queue_in.append((list(self), other))
            return True
        if isinstance(other, int) and (is_1d_list(self, Variable) or is_1d_tuple(self, Variable)):  # member/element constraint
            queue_in.append((self, other))
            return True
        return self.__contains__(other)

    def _list_contains(self, other):  # for being able to use 'in' when expressing extension constraints
        if not OpOverrider.activated:
            return self.__contains__(other)
        if isinstance(other, types.GeneratorType):
            other = list(other)
        if is_containing(other, Variable) and len(self) > 0 and isinstance(self[0], (list, tuple, int)):
            queue_in.append((self, other))
            return True
        if is_containing(other, Variable) and len(self) == 0:
            return other in set(self)
        error_if(is_containing(other, Variable),
                 "It seems that you should use a set and not a list, as in x in {...}." + " Your arguments are " + str(other) + " " + str(self))
        if isinstance(other, int) and (is_1d_list(self, Variable) or is_1d_tuple(self, Variable)):  # member/element constraint
            queue_in.append((self, other))
            return True
        return self.__contains__(other)

    def _set_contains(self, other):  # for being able to use 'in' when expressing intension/extension constraints
        if not OpOverrider.activated:
            return self.__contains__(other)
        if isinstance(other, types.GeneratorType):
            other = list(other)
        tself = unique_type_in(self)
        # if isinstance(other, Variable) and len(self) > 0 and is_containing(self, int):  # unary table constraint
        if isinstance(other, Variable) and tself in {int, str}:  # unary table constraint
            queue_in.append((list(self), other))
            return True
        # if isinstance(other, (Variable, PartialConstraint)) or isinstance(other, (int, str)) and is_containing(self, Variable):  # intension constraint
        if isinstance(other, (Variable, PartialConstraint)) or isinstance(other, (int, str)) and tself and issubclass(tself, Variable):  # intension constraint
            queue_in.append((self, other))
            return True
        # if is_1d_tuple(other, Variable) or is_1d_list(other, Variable):  # non-unary table constraint
        #     queue_in.append((list(self), other))
        #     return True
        if isinstance(other, (tuple, list)) and is_containing(other, (Variable, Node)):  # non-unary table constraint
            ll = flatten(other)
            for i in range(len(ll)):  # we replace nodes by auxiliary variables if present
                if isinstance(ll[i], Node):
                    ll[i] = auxiliary().replace_node(ll[i])
                    # if is_containing(other, Variable):  # non-unary table constraint
            if unsafe_cache:
                if not hasattr(_set_contains, "cache"):
                    _set_contains.cache = {}
                if id(self) not in _set_contains.cache:
                    _set_contains.cache[id(self)] = list({tuple(v) if isinstance(v, types.GeneratorType) else v for v in self})
            queue_in.append((_set_contains.cache[id(self)] if unsafe_cache else list(self), ll))  # flatten(other)))
            return True
        return self.__contains__(other)

    def _range_contains(self, other):  # for being able to use 'in' when expressing conditions of constraints
        if not OpOverrider.activated:
            return range.__contains__(other)
        if isinstance(other, ScalarProduct):
            other = PartialConstraint(ConstraintSum(other.variables, other.coeffs, None))  # functions.Sum(other)
        if isinstance(other, Variable):  # unary table constraint (based on a range)
            queue_in.append((list(self), other))
            return True
        if isinstance(other, PartialConstraint):
            queue_in.append((self, other))
            return True
        return range.__contains__(self, other)

    def _enumerate_contains(self, other):
        if not OpOverrider.activated:
            return self.__contains__(other)
        if is_containing(other, Variable):
            tmp = list(self)
            if len(tmp) > 0 and isinstance(tmp[0], (tuple, int)):
                queue_in.append((tmp, other))
                return True
        return self.__contains__(other)

    curse(dict, "__add__", _dict_add)
    curse(tuple, "__mul__", _tuple_mul)
    curse(list, "__mul__", _list_mul)
    curse(tuple, "__contains__", _tuple_contains)
    curse(list, "__contains__", _list_contains)
    curse(set, "__contains__", _set_contains)
    curse(range, "__contains__", _range_contains)
    curse(enumerate, "__contains__", _enumerate_contains)


cursing()


class OpOverrider:
    activated = False

    @staticmethod
    def enable():
        OpOverrider.activated = True

        ListVar.__eq__ = OpOverrider.__eq__lv
        ListVar.__ne__ = OpOverrider.__ne__lv
        ListVar.__getitem__ = OpOverrider.__getitem__lv
        ListInt.__getitem__ = OpOverrider.__getitem__li
        ListInt.__contains__ = OpOverrider.__contains__li

        Variable.__eq__ = Node.__eq__ = OpOverrider.__eq__
        Variable.__ne__ = Node.__ne__ = OpOverrider.__ne__
        Variable.__lt__ = Node.__lt__ = OpOverrider.__lt__
        Variable.__le__ = Node.__le__ = OpOverrider.__le__
        Variable.__ge__ = Node.__ge__ = OpOverrider.__ge__
        Variable.__gt__ = Node.__gt__ = OpOverrider.__gt__

        Variable.__add__ = Node.__add__ = OpOverrider.__add__
        Variable.__radd__ = Node.__radd__ = OpOverrider.__radd__
        Variable.__sub__ = Node.__sub__ = OpOverrider.__sub__
        Variable.__rsub__ = Node.__rsub__ = OpOverrider.__rsub__
        Variable.__mul__ = Node.__mul__ = OpOverrider.__mul__
        Variable.__rmul__ = Node.__rmul__ = OpOverrider.__rmul__
        Variable.__pow__ = Node.__pow__ = OpOverrider.__pow__
        Variable.__mod__ = Node.__mod__ = OpOverrider.__mod__
        Variable.__floordiv__ = Node.__floordiv__ = OpOverrider.__floordiv__
        Variable.__rfloordiv__ = Node.__rfloordiv__ = OpOverrider.__rfloordiv__

        Variable.__and__ = Node.__and__ = OpOverrider.__and__
        Variable.__or__ = Node.__or__ = OpOverrider.__or__
        Variable.__invert__ = Node.__invert__ = OpOverrider.__invert__
        Variable.__xor__ = Node.__xor__ = OpOverrider.__xor__

    @staticmethod
    def disable():
        OpOverrider.activated = False

        ListVar.__eq__ = list.__eq__
        ListVar.__ne__ = list.__ne__
        ListVar.__getitem__ = list.__getitem__
        ListInt.__getitem__ = list.__getitem__
        ListInt.__contains__ = list.__contains__

        Variable.__eq__ = Node.__eq__ = object.__eq__
        Variable.__ne__ = Node.__ne__ = object.__ne__
        Variable.__lt__ = Node.__lt__ = object.__lt__
        Variable.__le__ = Node.__le__ = object.__le__
        Variable.__ge__ = Node.__ge__ = object.__ge__
        Variable.__gt__ = Node.__gt__ = object.__gt__

        Variable.__add__ = Node.__add__ = None
        Variable.__radd__ = Node.__radd__ = None
        Variable.__sub__ = Node.__sub__ = None
        Variable.__rsub__ = Node.__rsub__ = None
        Variable.__mul__ = Node.__mul__ = None
        Variable.__rmul__ = Node.__rmul__ = None
        Variable.__pow__ = Node.__pow__ = None
        Variable.__mod__ = Node.__mod__ = None
        Variable.__floordiv__ = Node.__floordiv__ = None
        Variable.__rfloordiv__ = Node.__rfloordiv__ = None

        Variable.__and__ = Node.__and__ = None
        Variable.__or__ = Node.__or__ = None
        Variable.__invert__ = Node.__invert__ = None
        Variable.__xor__ = Node.__xor__ = None

        return OpOverrider

    @staticmethod
    def execute(arg):
        OpOverrider.enable()
        return arg

    @staticmethod
    def project_recursive(t, indexes, dimension):
        index = slice(None, None, None) if indexes[dimension] == ANY else indexes[dimension]
        if isinstance(index, int):
            if isinstance(t, list):
                t = t[index]  #  to keep the shape (dimensions), we need to do that
            else:
                return t
        elif isinstance(index, slice):
            t = list.__getitem__(t, index)
        else:
            raise TypeError()
        if isinstance(t, list) and dimension + 1 < len(indexes):
            if not isinstance(index, int):
                for i, element in enumerate(t):
                    t[i] = OpOverrider.project_recursive(element, indexes, dimension + 1)
            else:
                t = OpOverrider.project_recursive(t, indexes, dimension + 1)
        return t

    def __add__(self, other):
        if isinstance(other, PartialConstraint):
            return PartialConstraint.combine_partial_objects(self, TypeNode.ADD, other)
        return Node.build(TypeNode.ADD, self, other)

    def __radd__(self, other):
        return Node.build(TypeNode.ADD, other, self)

    def __sub__(self, other):
        if isinstance(other, PartialConstraint):
            return PartialConstraint.combine_partial_objects(self, TypeNode.SUB, other)
        return Node.build(TypeNode.SUB, self, other)

    def __rsub__(self, other):
        return Node.build(TypeNode.SUB, other, self)

    def __mul__(self, other):
        return Node.build(TypeNode.MUL, self, other)

    def __rmul__(self, other):
        return Node.build(TypeNode.MUL, other, self)

    def __mod__(self, other):
        return Node.build(TypeNode.MOD, self, other)

    def __pow__(self, other):
        return Node.build(TypeNode.POW, self, other)

    def __floordiv__(self, other):
        return Node.build(TypeNode.DIV, self, other)

    def __rfloordiv__(self, other):
        return Node.build(TypeNode.DIV, other, self)

    def __lt__(self, other):
        if self is None or other is None:
            return object.__lt__(self, other)
        return PartialConstraint.__gt__(other, self) if isinstance(other, PartialConstraint) else Node.build(TypeNode.LT, self, other)

    def __le__(self, other):
        if self is None or other is None:
            return object.__le__(self, other)
        return PartialConstraint.__ge__(other, self) if isinstance(other, PartialConstraint) else Node.build(TypeNode.LE, self, other)

    def __ge__(self, other):
        if self is None or other is None:
            return object.__ge__(self, other)
        if isinstance(other, int) and other == 1 and isinstance(self, Node) and self.type == TypeNode.DIST:  # we simplify the expression
            return Node.build(TypeNode.NE, self.sons[0], self.sons[1])
        return PartialConstraint.__le__(other, self) if isinstance(other, PartialConstraint) else Node.build(TypeNode.GE, self, other)

    def __gt__(self, other):
        if self is None or other is None:
            return object.__gt__(self, other)
        if isinstance(other, int) and other == 0 and isinstance(self, Node) and self.type == TypeNode.DIST:  # we simplify the expression
            return Node.build(TypeNode.NE, self.sons[0], self.sons[1])
        return PartialConstraint.__lt__(other, self) if isinstance(other, PartialConstraint) else Node.build(TypeNode.GT, self, other)

    def __eq__(self, other):
        if self is None or other is None:
            return object.__eq__(self, other)
        if isinstance(other, int) and other == 0 and isinstance(self, Node) and self.type == TypeNode.DIST:  # we simplify the expression
            return Node.build(TypeNode.EQ, self.sons[0], self.sons[1])
        return PartialConstraint.__eq__(other, self) if isinstance(other, PartialConstraint) else Node.build(TypeNode.EQ, self, other)

    def __ne__(self, other):
        if self is None or other is None:
            return object.__ne__(self, other)
        if isinstance(other, int) and other == 0 and isinstance(self, Node) and self.type == TypeNode.DIST:  # we simplify the expression
            return Node.build(TypeNode.NE, self.sons[0], self.sons[1])
        return PartialConstraint.__ne__(other, self) if isinstance(other, PartialConstraint) else Node.build(TypeNode.NE, self, other)

    def __or__(self, other):
        return object.__or__(self, other) if None in {self, other} else Node.disjunction(self, other)

    def __and__(self, other):
        return object.__and__(self, other) if None in {self, other} else Node.conjunction(self, other)

    def __invert__(self):
        return NotVariable(self) if isinstance(self, VariableInteger) else Node.build(TypeNode.NOT, self)

    def __xor__(self, other):
        return object.__xor__(self, other) if None in {self, other} else Node.build(TypeNode.XOR, self, other)

    def __eq__lv(self, other):  # lv for ListVar
        def Instantiation(*, variables, values):
            variables = flatten(variables)
            values = flatten(values) if not isinstance(values, range) else list(values)
            checkType(variables, [Variable])
            checkType(values, (int, [int]))
            if len(variables) == 0:
                return None
            if len(values) == 1 and len(variables) > 1:
                values = [values[0]] * len(variables)
            return ConstraintInstantiation(variables, values)

        if isinstance(other, (list, tuple)) and any(isinstance(v, int) for v in other):
            return ECtr(Instantiation(variables=self, values=other))
        return list.__eq__(self, other)

    def __ne__lv(self, other):  # lv for ListVar
        if isinstance(other, (list, tuple)) and any(isinstance(v, int) for v in other):
            return self not in {tuple(other)}
        return list.__ne__(self, other)

    def __getitem__lv(self, indexes):
        if isinstance(indexes, PartialConstraint):
            indexes = auxiliary().replace_partial_constraint(indexes)
        if isinstance(indexes, Variable):
            return PartialConstraint(ConstraintElement(self, indexes))
        if isinstance(indexes, tuple) and len(indexes) > 0:
            indexes = auxiliary().replace_partial_constraints(list(indexes))
            if any(isinstance(i, Variable) for i in indexes):  # this must be a constraint Element-Matrix
                assert is_matrix(self) and len(indexes) == 2, "A matrix is expected, with two indexes"
                if all(isinstance(i, Variable) for i in indexes):
                    return PartialConstraint(ConstraintElementMatrix(self, indexes[0], indexes[1]))
                else:
                    if isinstance(indexes[0], Variable) and isinstance(indexes[1], int):
                        return PartialConstraint(ConstraintElement(self[:, indexes[1]], indexes[0]))
                    elif isinstance(indexes[0], int) and isinstance(indexes[1], Variable):
                        return PartialConstraint(ConstraintElement(self[indexes[0]], indexes[1]))
                    else:
                        assert False
            result = OpOverrider.project_recursive(self, indexes, 0)
            try:
                return ListVar(result)  # TODO are sublists also guaranteed to be ListVar?
            except TypeError:
                return result
        result = list.__getitem__(self, indexes)
        try:
            return ListVar(result)
        except TypeError:
            return result

    def __getitem__li(self, indexes):  # li for ListInt
        if isinstance(indexes, PartialConstraint):
            indexes = auxiliary().replace_partial_constraint(indexes)
        if isinstance(indexes, Variable):
            return PartialConstraint(ConstraintElement(self, indexes))
        if isinstance(indexes, tuple) and len(indexes) > 0:
            indexes = auxiliary().replace_partial_constraints(list(indexes))
            if any(isinstance(i, Variable) for i in indexes):  # this must be a constraint Element-Matrix
                assert is_matrix(self) and len(indexes) == 2, "A matrix is expected, with two indexes"
                if all(isinstance(i, Variable) for i in indexes):
                    return PartialConstraint(ConstraintElementMatrix(self, indexes[0], indexes[1]))
                else:
                    if isinstance(indexes[0], Variable) and isinstance(indexes[1], int):
                        return PartialConstraint(ConstraintElement(self[:, indexes[1]], indexes[0]))
                    elif isinstance(indexes[0], int) and isinstance(indexes[1], Variable):
                        return PartialConstraint(ConstraintElement(self[indexes[0]], indexes[1]))
                    else:
                        assert False
            result = OpOverrider.project_recursive(self, indexes, 0)
            try:
                return ListVar(result)  # TODO is it ListVar or ListInt ?
            except TypeError:
                return result
        result = list.__getitem__(self, indexes)
        try:
            return ListInt(result)
        except TypeError:
            return result

    def __contains__li(self, other):
        if is_containing(other, Variable) and len(self) > 0 and isinstance(self[0], (tuple, int)):
            queue_in.append((self, other))
            return True
        return list.__contains__(self, other)


class ListInt(list):
    def __init__(self, integers):
        super().__init__(integers)  # self.extend(integers)

    def __getslice__(self, i, j):
        return ListInt(super().__getslice__(i, j))

    def __add__(self, other):
        return ListInt(super().__add__(other))

    def __mul__(self, other):
        if is_containing(other, (Variable, Node)):
            return ScalarProduct(other, self)
        assert is_containing(self, (Variable, Node))
        return ScalarProduct(self, other)

    def __rmul__(self, other):
        return ListInt.__mul__(other, self)


def columns(m):
    assert is_matrix(m), "columns() can only be called on matrices"
    return ListVar(ListVar(row[j] for row in m) for j in range(len(m[0])))


class ListVar(list):
    # def __new__(self, variables):  # if we subclass tuple instead of list (while removing __init__)
    #     return super().__new__(ListVar, variables)

    def __init__(self, variables):
        super().__init__(variables)

    def __getslice__(self, i, j):
        return ListVar(super().__getslice__(i, j))

    def __add__(self, other):
        return ListVar(super().__add__(other))

    def __mul__(self, other):
        assert is_containing(self, (Variable, Node))
        return ScalarProduct(self, list(other) if isinstance(other, (tuple, range)) else other)

    def __contains__(self, other):
        if isinstance(other, int) and (is_1d_list(self, Variable) or is_1d_tuple(self, Variable)):  # member constraint
            queue_in.append((self, other))
            return True
        return list.__contains__(self, other)

    # def __rmul__(self, other): return ListVar.__mul__(other, self)

    def columns(self):
        return columns(self)


def convert_to_namedtuples(obj):
    if not hasattr(convert_to_namedtuples, "cnt"):
        convert_to_namedtuples.cnt = 0
    if isinstance(obj, tuple):
        obj = list(obj)  # because if data come from a text file (and not from a JSON file), we have different structures, which leads to problems
    if isinstance(obj, list):
        if is_1d_list(obj, int):
            return ListInt(obj)
        if is_1d_list(obj, Variable):
            return ListVar(obj)
        if is_1d_list(obj, dict):
            nt = namedtuple("nt" + str(convert_to_namedtuples.cnt), obj[0].keys())
            convert_to_namedtuples.cnt += 1
            return [nt(*(convert_to_namedtuples(v) for (k, v) in d.items())) for d in obj]
        t = [convert_to_namedtuples(v) for v in obj]
        return ListInt(t) if isinstance(t[0], ListInt) else ListVar(t) if isinstance(t[0], ListVar) else t
    if isinstance(obj, dict):
        nt = namedtuple("nt" + str(convert_to_namedtuples.cnt), obj.keys())
        convert_to_namedtuples.cnt += 1
        return nt(*(convert_to_namedtuples(v) for (k, v) in obj.items()))
    return obj


def is_namedtuple(obj):  # imperfect way of checking, but must be enough for our use (when JSON dumping Compilation.data)
    t = type(obj)
    if len(t.__bases__) != 1 or t.__bases__[0] != tuple:
        return False
    fields = getattr(t, '_fields', None)
    return isinstance(fields, tuple) and all(type(field) == str for field in fields)



"""
See https://en.wikipedia.org/wiki/Vehicle_routing_problem
This model is similar to the one proposed by Jakob Puchinger for the 2009 MiniZinc competition

Example of Execution:
  python3 vrp.py -data=Vrp_P-n16-k8.json
"""

from pycsp3 import *

n, capacity, demand, distances = data

# x[i][j] is 1 iff the arc (i,j) is part of a route
x = VarArray(size=[n, n], dom=lambda i, j: {0} if i == j else {0, 1})

# u[i] is the vehicle load after visiting the ith node (used for subtour elimination)
u = VarArray(size=n, dom=lambda i: {0} if i == 0 else range(capacity + 1))

satisfy(
    # exactly one incoming arc for each node j other than the depot (node 0)
    [Count(x[:, j], value=1) == 1 for j in range(1, n)],

    # exactly one outgoing arc for each node i other than the depot (node 0)  
    [Count(x[i], value=1) == 1 for i in range(1, n)],

    # Miller-Tucker-Zemlin subtour elimination 
    [[u[i], u[j], x[i][j]] * [1, -1, capacity] <= capacity - demand[j] for i in range(1, n) for j in range(1, n) if i != j],

    # satisfying demand at each node
    [u[i] >= demand[i] for i in range(1, n)]
)

minimize(
    x * distances
)



"""
This problem has been conceived by Olivier Dubois, and submitted to the second DIMACS Implementation Challenge.
Dubois's generator produces contradictory 3-SAT instances that seem very difficult to be solved by any general method.
Given an integer n, called the degree, Dubois's process allows us to construct a 3-SAT contradictory instance with 3 * n variables and 2 * n clauses,
each of them having 3 literals.

Examples of Execution:
  python3 Dubois.py
  python3 Dubois.py -data=10
"""
from pycsp3 import *

n = data or 8

table1 = {(0, 0, 1), (0, 1, 0), (1, 0, 0), (1, 1, 1)}
table2 = {(0, 0, 0), (0, 1, 1), (1, 0, 1), (1, 1, 0)}

x = VarArray(size=3 * n, dom={0, 1})

satisfy(
    (x[2 * n - 2], x[2 * n - 1], x[0]) in table1,

    [(x[i], x[2 * n + i], x[i + 1]) in table1 for i in range(n - 2)],

    [(x[n - 2 + i], x[3 * n - 2], x[3 * n - 1]) in table1 for i in range(2)],

    [(x[i], x[4 * n - 3 - i], x[i - 1]) in table1 for i in range(n, 2 * n - 2)],

    (x[2 * n - 2], x[2 * n - 1], x[2 * n - 3]) in table2
)



"""
See https://en.wikipedia.org/wiki/Graph_coloring

Examples of Execution:
  python3 Coloring.py -data=Coloring_rand1.json
  python3 Coloring.py -data=Coloring_rand1.json -variant=csp
"""

from pycsp3 import *

nNodes, nColors, edges = data

# x[i] is the color assigned to the ith node of the graph
x = VarArray(size=nNodes, dom=range(nColors))

satisfy(
    # two adjacent nodes must be colored differently
    x[i] != x[j] for (i, j) in edges
)

if not variant():
    minimize(
        # minimizing the greatest used color index (and, consequently, the number of colors)
        Maximum(x)
    )

elif variant("csp"):
    satisfy(
        # tag(symmetry-breaking)
        x[i] <= i for i in range(min(nNodes, nColors))
    )



"""
See https://en.wikipedia.org/wiki/Latin_square

Examples of Execution:
  python3 LatinSquare.py -data=[8,]
  python3 LatinSquare.py -data=LatinSquare_qwh-o030-h320.json
"""

from pycsp3 import *

n, clues = data  # if not -1, clues[i][j] is a value imposed at row i and col j

# x[i][j] is the value at row i and col j of the Latin Square
x = VarArray(size=[n, n], dom=range(n))

satisfy(
    AllDifferent(x, matrix=True),

    # tag(clues)
    [x[i][j] == clues[i][j] for i in range(n) for j in range(n) if clues and clues[i][j] != -1]
)



"""
Problem 053 on CSPLib

Examples of Execution:
  python3 GracefulGraph.py
  python3 GracefulGraph.py -data=[3,5]
"""

from pycsp3 import *

k, p = data or (2, 4)  # k is the size of each clique K (number of nodes) -- p is the size of each path P (or equivalently, number of cliques)
nEdges = int(((k * (k - 1)) * p) / 2 + k * (p - 1))

# cn[i][j] is the color of the jth node of the ith clique
cn = VarArray(size=[p, k], dom=range(nEdges + 1))

# ce[i][j1][j2] is the color of the edge (j1,j2) of the ith clique, for j1 strictly less than j2
ce = VarArray(size=[p, k, k], dom=lambda i, j1, j2: range(1, nEdges + 1) if j1 < j2 else None)

# cp[i][j] is the color of the jth edge of the ith path
cp = VarArray(size=[p - 1, k], dom=range(1, nEdges + 1))

satisfy(
    # all nodes are colored differently
    AllDifferent(cn),

    # all edges are colored differently
    AllDifferent(ce + cp),

    # computing colors of edges from colors of nodes
    [
        [ce[i][j1][j2] == dist(cn[i][j1], cn[i][j2]) for i in range(p) for j1, j2 in combinations(range(k), 2)],

        [cp[i][j] == dist(cn[i][j], cn[i + 1][j]) for i in range(p - 1) for j in range(k)]
    ]
)
